﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Globalization;
using System.Linq;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;
using System.Runtime.Serialization.Formatters.Binary;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {
        static readonly char[] _separator = new char[] { '=' };
        private static object _sync = new object();

        private static StringBuilder _sb;
        public static string Append(this String mystring, params string[] s)
        {
            lock (_sync)
            {
                if (_sb == null)
                    _sb = new StringBuilder();
                else
                    _sb.Clear();

                _sb.Append(mystring);
                foreach (var ss in s)
                    _sb.Append(ss);

                return _sb.ToString();
            }
        }

        public static List<string> GetSemicolonSeparatedValues(string input)
        {
            var ret = new List<string>();
            if (!input.Contains(";"))
            {
                var inputTrimmed = input.Trim();
                if (!string.IsNullOrEmpty(inputTrimmed)) ret.Add(inputTrimmed);
                return ret;
            }
            var subjectCandidates = input.Split(new char[] { ';' });
            foreach (var sc in subjectCandidates)
            {
                var sct = sc.Trim();
                if (sc.Length > 0) ret.Add(sct);
            }
            return ret;
        }

        public static string GetSubstringByString(string start, string end, string total)
        {
            var indexA = total.IndexOf(start);
            if (indexA == -1) return null;

            var indexB = total.IndexOf(end, indexA);
            if (indexB == -1)
                indexB = total.Length;

            return total.Substring((indexA + start.Length), (indexB - indexA - start.Length));
        }

        public static string ReplaceFirst(string text, string search, string replace)
        {
            int pos = text.IndexOf(search);
            if (pos < 0)
            {
                return text;
            }
            return text.Substring(0, pos) + replace + text.Substring(pos + search.Length);
        }

        public static string AddTrailing(string str, string trailing)
        {
            if (str.EndsWith(trailing)) return str;
            return str.Append(trailing);
        }

        public static string RemoveTailEndChar(string s)
        {
            return s.TrimEnd(new char[] { '\0' });
        }

        public static string RemoveTailSlash(string s)
        {
            return s.TrimEnd(new char[] { '\\', '/' });
        }

        public const string WinNewLine = "\r\n";

        public static object Set(object obj, object val)
        {
            obj = val;
            return obj;
        }

        public static string SetString(object obj, string val)
        {
            obj = val;
            return val;
        }

        public static NumberFormatInfo DecimalPointdouble
        {
            get
            {
                var ci = CultureInfo.InstalledUICulture;
                var ni = (NumberFormatInfo)ci.NumberFormat.Clone();
                ni.NumberDecimalSeparator = ".";
                return ni;
            }
        }


        public static string Substring(string s, int maxlen)
        {
            if (string.IsNullOrEmpty(s)) return null;
            if (s.Length <= maxlen)
            {
                return s;
            }

            return s.Substring(0, maxlen) + "...";
        }

        public static int IndexOfAny(string test, string[] values, out string match)
        {
            match = "";
            int first = -1;
            foreach (string item in values)
            {
                int i = test.IndexOf(item);
                if (i < 0) continue;
                if (i == 0)
                {
                    match = item;
                    return 0;
                }

                if ((i < first) || (first == -1))
                {
                    first = i;
                    match = item;
                }
            }
            return first;
        }

        public static string ParseBackSpace(string s)
        {
            var ret = new StringBuilder(String.Empty);
            int x = 0;
            int l = s.Length;
            while (x < l)
            {
                if (s[x] == Backspace)
                {
                    ret.Remove(ret.Length - 1, 1);
                }
                else if (s[x] >= Sp)
                {
                    ret.Append(s[x]);
                }
                x++;
            }
            return ret.ToString();
        }


        public static ulong Ordstr(string s)
        {
            ulong ret = 0;
            for (var x = 0; x < s.Length; x++)
            {
                ret = (ret << 8) + (byte)s[x];
            }
            return ret;
        }


        public static string PushToString(int i)
        {
            return PushToString((ulong)i);
        }

        public static string PushToString(ulong i)
        {
            var ret = String.Empty;
            while (i > 0)
            {
                var c = (byte)(i & 0xFF);
                i = i >> 8;
                ret = (char)c + ret;
            }

            return (ret);
        }

        public static string ToNumber(string s)
        {
            return ToNumberEx(s, false);
        }

        public static string ToNumberEx(string s, bool plusSignsAllowed)
        {
            if (s == null) return "";
            var ret = String.Empty;
            for (var x = 0; x < s.Length; x++)
            {
                if (((s[x] >= '0') && (s[x] <= '9')) ||
                    ((plusSignsAllowed) && (s[x] == '+'))) ret = ret + s[x];
            }

            if (ret == string.Empty)
            {
                ret = "0";
            }
            return ret;
        }

        public static string ToSingleLine(string input)
        {
            input = input.Replace("\n", string.Empty);
            input = input.Replace("\r", string.Empty);
            return input;
        }

        public static string ToFakeSingleLine(string input)
        {
            input = input.Replace("\n", @"\n");
            input = input.Replace("\r", @"\r");
            return input;
        }

        public static bool ContainsKey(NameValueCollection nvc, string key)
        {
            string compTo = key.ToLower(CultureInfo.InvariantCulture);

            foreach (string nvckey in nvc.Keys)
            {
                if ((nvckey != null) && (nvckey.ToLower(CultureInfo.InvariantCulture) == compTo))
                {
                    return true;
                }
            }
            return false;
        }

  

        public static string NameValueCollectionToString(NameValueCollection nvc)
        {
            var ret = new StringBuilder();
            var simplified = SimplifyNameValueCollection(nvc, Encoding.UTF8);
            foreach (var kvp in simplified)
            {
                if (ret.Length > 0) ret.Append("&");
                ret.Append(kvp.Key + "=" + kvp.Value);
            }
            return ret.ToString();
        }

        public static Dictionary<string, string> SimplifyNameValueCollection(NameValueCollection nvc, Encoding encoding)
        {
            Dictionary<string, string> result = new Dictionary<string, string>();

            foreach (string nvckey in nvc.Keys)
            {
                if (nvckey == null) continue;

                string[] values = nvc.GetValues(nvckey);
                string primaryvalue = String.Empty;
                if (values != null && values.Length > 0) { primaryvalue = values[0]; }

                QueryStringToExistingDictionary(primaryvalue, result, encoding);

                string r = nvckey;
                var key = EncodeToUTF8(r);

                if (!result.ContainsKey(key))
                    //result.Add(key, EncodeToUTF8(primaryvalue));
                    result.Add(key, primaryvalue);
            }
            return result;
        }

        public static string EncodeToUTF8(string temp)
        {
            byte[] bytes = Encoding.Default.GetBytes(temp);
            return Encoding.UTF8.GetString(bytes);
        }

        public static string DecodeFromUTF8(string temp)
        {
            byte[] bytes = Encoding.UTF8.GetBytes(temp);
            return Encoding.Default.GetString(bytes);
        }

        public static string HumanReadable(Dictionary<string, string> stringDictionary)
        {
            var ret = new StringBuilder();
            foreach (KeyValuePair<string, string> kvp in stringDictionary)
            {
                var innerBuilder = new StringBuilder();
                innerBuilder.Append(kvp.Key).Append(": '").Append(kvp.Value).Append("'; ");
                ret.Append(innerBuilder);
            }
            return ret.ToString();
        }

        public static IEnumerable<string> SplitToLines(string text)
        {
            return Regex.Split(text, "\r\n|\r|\n");
        }

        public static IEnumerable<string> Split(string str, int chunkSize)
        {
            var chunks = new List<string>();
            for (int i = 0; i < str.Length; i += chunkSize)
                chunks.Add(str.Substring(i, Math.Min(chunkSize, str.Length - i)));
            return chunks;
        }



        public static long ObjectSize(object o)
        {
            long size = 0;

            using (Stream s = new MemoryStream())
            {
                BinaryFormatter formatter = new BinaryFormatter();
                formatter.Serialize(s, o);
                size = s.Length;
            }
            return size;
        }

        public static string h2e(string s)
        {
            char[,] kar = new char[,]{
                {'á','Á','é','É','í','Í','ú','Ú','ü','Ü','ű','Ű','ó','Ó','ö','Ö','ő','Ő'},
                {'a','A','e','E','i','I','u','U','u','U','u','U','o','O','o','O','o','O'},
            };

            for (int i = 0; i < s.Length; i++)
            {
                for (int j = 0; j < 18; j++)
                {
                    if (s[i] == kar[0, j])
                    {
                        s = s.Replace(s[i], kar[1, j]);
                    }
                }

            }
            return s;
        }

        public static string Truncate(string s, int maxLength)
        {
            if (string.IsNullOrEmpty(s))
                return "";
            if (maxLength < 3)
                return s.Substring(0, maxLength);
            if (s.Length <= maxLength)
                return s;
            return s.Substring(0, maxLength - 3) + "...";
        }

        public static string NullIfWhiteSpace(string s)
        {
            if (string.IsNullOrWhiteSpace(s))
                return null;
            return s;
        }

        public static string ReplaceIfWhiteSpace(string s, string replacement)
        {
            if (string.IsNullOrWhiteSpace(s))
                return replacement;
            return s;
        }
    }
}