﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {
        public static byte[] StringToByteArray(string instr)
        {
            var byteArr = new byte[instr.Length];
            for (var i = 0; i < instr.Length; i++)
                byteArr[i] = (byte)(instr[i] & 255);
            return byteArr;
        }

        public static byte[] GetBytes(string str)
        {
            byte[] bytes = new byte[str.Length * sizeof(char)];
            Buffer.BlockCopy(str.ToCharArray(), 0, bytes, 0, bytes.Length);
            return bytes;
        }

      
        public static byte[] SerializeByteList(List<byte[]> bl)
        {

            int cap = 4;
            foreach (var b in bl)
            {
                cap += b.Length + 4;
            }
            var ret = new byte[cap];

            int pos = 0;
            var cnt = IntToBytes(bl.Count);
            Buffer.BlockCopy(cnt, 0, ret, pos, 4);
            pos += 4;

            for (int x = 0; x < bl.Count; x++)
            {
                var b = bl[x];
                var lenBytes = OzCommonStrings.IntToBytes(b.Length);
                Buffer.BlockCopy(lenBytes, 0, ret, pos, 4);
                Buffer.BlockCopy(b, 0, ret, pos + 4, b.Length);
                pos += b.Length + 4;
            }
            return ret;
        }

        public static List<byte[]> DeserializeByteList(byte[] bytes)
        {
            var cntBytes = new byte[4];
            Buffer.BlockCopy(bytes, 0, cntBytes, 0, 4);
            var cnt = BytesToInt(cntBytes);
            int pos = 4;

            var ret = new List<byte[]>(cnt);
            for (int x = 0; x < cnt; x++)
            {
                Buffer.BlockCopy(bytes, pos, cntBytes, 0, 4);
                var len = OzCommonStrings.BytesToInt(cntBytes);
                pos += 4;

                var data = new byte[len];
                Buffer.BlockCopy(bytes, pos, data, 0, len);
                pos += len;
                ret.Add(data);
            }
            return ret;
        }

    

        //Ezt csak olyan stringből való dekódolásra szabad használni, amit a GetStringFromBytes1-ból állítottunk elő
        public static byte[] GetBytes1(string str)
        {
            bool even = str[0] == '0';
            byte[] bytes = new byte[(str.Length - 1) * sizeof(char) + (even ? 0 : -1)];
            char[] chars = str.ToCharArray();
            Buffer.BlockCopy(chars, 2, bytes, 0, bytes.Length);

            return bytes;
        }

        public static string GetStringFromBytes1(byte[] bytes)
        {
            bool even = (bytes.Length % 2 == 0);
            char[] chars = new char[1 + bytes.Length / sizeof(char) + (even ? 0 : 1)];
            chars[0] = (even ? '0' : '1');
            Buffer.BlockCopy(bytes, 0, chars, 2, bytes.Length);
            return new string(chars);
        }

        public static string GetStringFromUcs2Bytes(byte[] bytes)
        {
            int charCount = bytes.Length / 2;
            var sb = new StringBuilder(charCount);
            for (int x = 0; x < charCount; x++)
            {
                var y = x * 2;
                var b = new byte[2];
                b[0] = bytes[y+1];
                b[1] = bytes[y];
                var cs = System.BitConverter.ToChar(b, 0);
                sb.Append(cs);
            }
            return sb.ToString();
        }

 

        public static string GetString(byte[] bytes)
        {
            char[] chars = new char[bytes.Length / sizeof(char)];
            Buffer.BlockCopy(bytes, 0, chars, 0, bytes.Length);
            return new string(chars);
        }

        public static List<byte[]> ByteSplit(byte[] source, int segmentSize)
        {
            int segmentCount = (int)(source.Length / segmentSize);
            int remainder = source.Length % segmentSize;
            int totalSegments = segmentCount;
            if (remainder > 0) totalSegments++;
            var ret = new List<byte[]>(totalSegments);
            for (int x = 0; x < segmentCount; x ++)
            {
                var b = new byte[segmentSize];
                Buffer.BlockCopy(source, x * segmentSize, b, 0, segmentSize);
                ret.Add(b);
            }
            if (remainder > 0)
            {
                var b = new byte[remainder];
                Buffer.BlockCopy(source, segmentCount * segmentSize, b, 0, remainder);
                ret.Add(b);
            }
            return ret;
        }

        public static byte[] ByteSubString(byte[] src, int start, int length = -1)
        {
            int srcLen = src.Length;
            if (start > srcLen) return new byte[0];
            if (length < -1) length = -1;
            if ((length == -1) || (start + length > srcLen)) { length = srcLen - start; }

            byte[] ret = new byte[length];
            Buffer.BlockCopy(src, start, ret, 0, length);
            return ret;
        }

        public static void StringToTwoByteArray(string theString, ref byte[] data, int offset)
        {
            for (var i = 0; i < theString.Length; ++i)
            {
                var realI = (i * 2) + offset;
                data[realI] = (byte)(theString[i] & 0xFF);
                data[realI + 1] = (byte)((theString[i] & 0xFF00) >> 16);
            }
        }

        public static void StringToByteArray(string theString, ref byte[] data, int offset)
        {
            for (int i = 0; i < theString.Length; ++i)
                data[i] = (byte)(theString[i] & 0xFF);
        }

        public static string ByteArrayToString(byte[] data, int count)
        {
            var ret = new StringBuilder(String.Empty);

            for (var x = 0; x < count; x++)
                ret.Append((char)data[x]);

            return ret.ToString();
        }

        public static string ByteArrayToString(byte[] data)
        {
            return ByteArrayToString(data, data.Length);
        }

        public static char MakeByteFromBits(bool bit7, bool bit6, bool bit5, bool bit4, bool bit3, bool bit2, bool bit1,
                                    bool bit0)
        {
            int ret = (BoolToInt(bit7) << 7) +
                      (BoolToInt(bit6) << 6) +
                      (BoolToInt(bit5) << 5) +
                      (BoolToInt(bit4) << 4) +
                      (BoolToInt(bit3) << 3) +
                      (BoolToInt(bit2) << 2) +
                      (BoolToInt(bit1) << 1) +
                      (BoolToInt(bit0));
            return (char)ret;
        }

        public static byte[] AppendByteArrays(byte[] a1, byte[] a2)
        {
            byte[] rv = new byte[a1.Length + a2.Length];
            Buffer.BlockCopy(a1, 0, rv, 0, a1.Length);
            Buffer.BlockCopy(a2, 0, rv, a1.Length, a2.Length);
            return rv;
        }

        public static byte[] ConcatByteArrays(List<byte[]> arrays)
        {
            return ConcatByteArrays(arrays.ToArray());
        }

        public static byte[] ConcatByteArrays(params byte[][] arrays)
        {
            try
            {
                byte[] rv = new byte[arrays.Sum(a => a.Length)];
                int offset = 0;
                foreach (byte[] array in arrays)
                {
                    Buffer.BlockCopy(array, 0, rv, offset, array.Length);
                    offset += array.Length;
                }
                return rv;
            }
            catch
            {
                return null;
            }
        }

        public static byte[] GetByteArraySubstring(byte[] array, int start, int length)
        {
            try
            {
                byte[] rv = new byte[length];
                Buffer.BlockCopy(array, start, rv, 0, length);
                return rv;
            }
            catch
            {
                return null;
            }
        }

        public static byte[] GetByteArraySubstring(byte[] array, long start, long length)
        {
            try
            {
                byte[] rv = new byte[length];
                for (long i = 0; i < length; i++)
                    rv[i] = array[i + start];
                return rv;
            }
            catch
            {
                return null;
            }
        }

        public static int GetIntFromExchangedByte(char c)
        {
            return GetIntFromExchangedByte((byte)c);
        }

        public static int GetIntFromExchangedByte(byte c)
        {
            try
            {
                var value = GetExchangedCharsFromByte(c);
                return int.Parse(value);
            }
            catch
            {
                return 0;
            }
        }

        public static string GetExchangedCharsFromByte(byte b)
        {
            string ret = "" + (char)((((byte)b) & 15) + '0');

            if (b < '\xF0') { ret += (char)((((byte)b) >> 4) + '0'); }

            return ret;
        }

        public static string GetStringIter(byte[] bytes, int length)
        {
            StringBuilder s = new StringBuilder();
            for (int x = 0; x < length; x++)
            {
                var c = (char)bytes[x];
                //Nem lépünk ki, mert jöhet olyan serialized adat, ami 0 byte-ot tartalmaz.
                //if (c == '\0') break;
                s.Append(c);
            }
            return s.ToString();
        }

        public static byte[] GetBytesIter(string str)
        {
            byte[] ret = new byte[str.Length];
            var length = str.Length;
            for (int x = 0; x < length; x++)
                ret[x] = (byte)str[x];
            return ret;
        }

        public static List<byte[]> SplitBytes(byte[] arr, byte splitchar)
        {
            var ret = new List<byte[]>();
            var buff = new byte[arr.Length];
            int cx = 0;
            for (int x = 0; x < arr.Length; x++)
            {
                if (arr[x] == splitchar)
                {
                    if (cx == 0)
                    {
                        ret.Add(new byte[0]); continue;
                    }
                    var dest = new byte[cx];
                    Buffer.BlockCopy(arr, x - cx, dest, 0, cx);
                    ret.Add(dest);
                    cx = 0;
                }
                cx++;
            }
            if (cx > 0)
            {
                var dest = new byte[cx - 1];
                Buffer.BlockCopy(arr, arr.Length - cx + 1, dest, 0, cx - 1);
                ret.Add(dest);
            }
            return ret;
        }

        public static byte[] RemoveTrailingZeroBytes(byte[] bi)
        {
            if (bi.Length == 0) return bi;
            if (bi[bi.Length - 1] != 0) return bi;
            int c = 0;
            while (c < bi.Length && bi[bi.Length - c - 1] == 0)
            {
                c++;
            }
            var bo = new byte[bi.Length - c];
            Buffer.BlockCopy(bi, 0, bo, 0, bi.Length - c);
            return bo;
        }
    }
}