﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {
        public static byte[] IntToBytes(int i)
        {
            byte[] buffer = BitConverter.GetBytes(i);
            if (BitConverter.IsLittleEndian)
                Array.Reverse(buffer);
            return buffer;
        }

        public static byte[] LongToBytes(long i)
        {
            byte[] buffer = BitConverter.GetBytes(i);
            if (BitConverter.IsLittleEndian)
                Array.Reverse(buffer);
            return buffer;
        }

        public static int BytesToInt(byte[] b)
        {
            if (BitConverter.IsLittleEndian)
                Array.Reverse(b);
            int i = BitConverter.ToInt32(b, 0);
            return i;
        }

        public static int BytesToInt(byte[] b, int startIndex)
        {
            var bs = new byte[4];
            Buffer.BlockCopy(b, startIndex, bs, 0, 4);

            if (BitConverter.IsLittleEndian)
                Array.Reverse(bs);
            int i = BitConverter.ToInt32(bs, 0);
            return i;
        }

        public static long BytesToLong(byte[] b)
        {
            if (BitConverter.IsLittleEndian)
                Array.Reverse(b);
            long i = BitConverter.ToInt64(b, 0);
            return i;
        }

        public static long BytesToLong(byte[] b, int startIndex)
        {
            var bs = new byte[8];
            Buffer.BlockCopy(b, startIndex, bs, 0, 8);

            if (BitConverter.IsLittleEndian)
                Array.Reverse(bs);
            long i = BitConverter.ToInt64(bs, 0);
            return i;
        }

        public static void ReadInt(byte[] bytes, ref int offset, out int value)
        {
            var b = new byte[4];
            Buffer.BlockCopy(bytes, offset, b, 0, b.Length);
            offset += b.Length;
            value = BytesToInt(b);
        }

        public static byte[] GetBytes(UInt16 uintValue)
        {
            byte[] dest = new byte[0];
            byte[] bytearray = BitConverter.GetBytes(uintValue);
            if (BitConverter.IsLittleEndian) Array.Reverse(bytearray);
            return bytearray;
        }

        /// <summary>
        /// Moves the int into a byte array, and pads the front with zeros
        /// </summary>
        /// <param name="uintValue"></param>
        /// <param name="length">Allowed values 0..maxint</param>
        /// <returns></returns>
        public static byte[] GetBytes(uint uintValue, int length = 4)
        {
            byte[] dest = new byte[0];
            byte[] bytearray = BitConverter.GetBytes(uintValue);
            if (BitConverter.IsLittleEndian) Array.Reverse(bytearray);
            if (length == 4) return bytearray;
            else if (length > 4)
            {
                dest = new byte[length];
                Buffer.BlockCopy(bytearray, 0, dest, length - 4, 4);
            }
            else if (length < 4)
            {
                dest = new byte[length];
                Buffer.BlockCopy(bytearray, 4 - length, dest, 0, length);
            }
            return dest;
        }

        public static byte[] GetBytes(int intValue)
        {
            byte[] intBytes = BitConverter.GetBytes(intValue);
            if (BitConverter.IsLittleEndian) Array.Reverse(intBytes);
            return intBytes;
        }

    }
}