﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Globalization;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {
        public enum CharSetInfo
        {
            // ReSharper disable InconsistentNaming
            CODEPAGE = 0,
            SHORTNAMES,
            LONGNAMES
            // ReSharper restore InconsistentNaming
        }

        private static readonly ArrayList CodePage = new ArrayList();
        private static readonly ArrayList ShortNames = new ArrayList();
        private static readonly ArrayList LongNames = new ArrayList();

        public static ArrayList CharSetNames(CharSetInfo listType)
        {
            if (CodePage.Count == 0)
            {
                #region Fill list

                CodePage.Add("Default");
                ShortNames.Add("Default");
                LongNames.Add("Default");
                CodePage.Add("37");
                ShortNames.Add("IBM037");
                LongNames.Add("IBM EBCDIC (US-Canada)");
                CodePage.Add("437");
                ShortNames.Add("IBM437");
                LongNames.Add("OEM United States");
                CodePage.Add("500");
                ShortNames.Add("IBM500");
                LongNames.Add("IBM EBCDIC (International)");
                CodePage.Add("708");
                ShortNames.Add("ASMO-708");
                LongNames.Add("Arabic (ASMO 708)");
                CodePage.Add("720");
                ShortNames.Add("DOS-720");
                LongNames.Add("Arabic (DOS)");
                CodePage.Add("737");
                ShortNames.Add("ibm737");
                LongNames.Add("Greek (DOS)");
                CodePage.Add("775");
                ShortNames.Add("ibm775");
                LongNames.Add("Baltic (DOS)");
                CodePage.Add("850");
                ShortNames.Add("ibm850");
                LongNames.Add("Western European (DOS)");
                CodePage.Add("852");
                ShortNames.Add("ibm852");
                LongNames.Add("Central European (DOS)");
                CodePage.Add("855");
                ShortNames.Add("IBM855");
                LongNames.Add("OEM Cyrillic");
                CodePage.Add("857");
                ShortNames.Add("ibm857");
                LongNames.Add("Turkish (DOS)");
                CodePage.Add("858");
                ShortNames.Add("IBM00858");
                LongNames.Add("OEM Multilingual Latin I");
                CodePage.Add("860");
                ShortNames.Add("IBM860");
                LongNames.Add("Portuguese (DOS)");
                CodePage.Add("861");
                ShortNames.Add("ibm861");
                LongNames.Add("Icelandic (DOS)");
                CodePage.Add("862");
                ShortNames.Add("DOS-862");
                LongNames.Add("Hebrew (DOS)");
                CodePage.Add("863");
                ShortNames.Add("IBM863");
                LongNames.Add("French Canadian (DOS)");
                CodePage.Add("864");
                ShortNames.Add("IBM864");
                LongNames.Add("Arabic (864)");
                CodePage.Add("865");
                ShortNames.Add("IBM865");
                LongNames.Add("Nordic (DOS)");
                CodePage.Add("866");
                ShortNames.Add("cp866");
                LongNames.Add("Cyrillic (DOS)");
                CodePage.Add("869");
                ShortNames.Add("ibm869");
                LongNames.Add("Greek, Modern (DOS)");
                CodePage.Add("870");
                ShortNames.Add("IBM870");
                LongNames.Add("IBM EBCDIC (Multilingual Latin-2)");
                CodePage.Add("874");
                ShortNames.Add("windows-874");
                LongNames.Add("Thai (Windows)");
                CodePage.Add("875");
                ShortNames.Add("cp875");
                LongNames.Add("IBM EBCDIC (Greek Modern)");
                CodePage.Add("932");
                ShortNames.Add("shift_jis");
                LongNames.Add("Japanese (Shift-JIS)");
                CodePage.Add("936");
                ShortNames.Add("gb2312");
                LongNames.Add("Chinese Simplified (GB2312)");
                CodePage.Add("949");
                ShortNames.Add("ks_c_5601-1987");
                LongNames.Add("Korean");
                CodePage.Add("950");
                ShortNames.Add("big5");
                LongNames.Add("Chinese Traditional (Big5)");
                CodePage.Add("1026");
                ShortNames.Add("IBM1026");
                LongNames.Add("IBM EBCDIC (Turkish Latin-5)");
                CodePage.Add("1047");
                ShortNames.Add("IBM01047");
                LongNames.Add("IBM Latin-1");
                CodePage.Add("1140");
                ShortNames.Add("IBM01140");
                LongNames.Add("IBM EBCDIC (US-Canada-Euro)");
                CodePage.Add("1141");
                ShortNames.Add("IBM01141");
                LongNames.Add("IBM EBCDIC (Germany-Euro)");
                CodePage.Add("1142");
                ShortNames.Add("IBM01142");
                LongNames.Add("IBM EBCDIC (Denmark-Norway-Euro)");
                CodePage.Add("1143");
                ShortNames.Add("IBM01143");
                LongNames.Add("IBM EBCDIC (Finland-Sweden-Euro)");
                CodePage.Add("1144");
                ShortNames.Add("IBM01144");
                LongNames.Add("IBM EBCDIC (Italy-Euro)");
                CodePage.Add("1145");
                ShortNames.Add("IBM01145");
                LongNames.Add("IBM EBCDIC (Spain-Euro)");
                CodePage.Add("1146");
                ShortNames.Add("IBM01146");
                LongNames.Add("IBM EBCDIC (UK-Euro)");
                CodePage.Add("1147");
                ShortNames.Add("IBM01147");
                LongNames.Add("IBM EBCDIC (France-Euro)");
                CodePage.Add("1148");
                ShortNames.Add("IBM01148");
                LongNames.Add("IBM EBCDIC (International-Euro)");
                CodePage.Add("1149");
                ShortNames.Add("IBM01149");
                LongNames.Add("IBM EBCDIC (Icelandic-Euro)");
                CodePage.Add("1200");
                ShortNames.Add("utf-16");
                LongNames.Add("Unicode");
                CodePage.Add("1201");
                ShortNames.Add("unicodeFFFE");
                LongNames.Add("Unicode (Big-Endian)");
                CodePage.Add("1250");
                ShortNames.Add("windows-1250");
                LongNames.Add("Central European (Windows)");
                CodePage.Add("1251");
                ShortNames.Add("windows-1251");
                LongNames.Add("Cyrillic (Windows)");
                CodePage.Add("1252");
                ShortNames.Add("Windows-1252");
                LongNames.Add("Western European (Windows)");
                CodePage.Add("1253");
                ShortNames.Add("windows-1253");
                LongNames.Add("Greek (Windows)");
                CodePage.Add("1254");
                ShortNames.Add("windows-1254");
                LongNames.Add("Turkish (Windows)");
                CodePage.Add("1255");
                ShortNames.Add("windows-1255");
                LongNames.Add("Hebrew (Windows)");
                CodePage.Add("1256");
                ShortNames.Add("windows-1256");
                LongNames.Add("Arabic (Windows)");
                CodePage.Add("1257");
                ShortNames.Add("windows-1257");
                LongNames.Add("Baltic (Windows)");
                CodePage.Add("1258");
                ShortNames.Add("windows-1258");
                LongNames.Add("Vietnamese (Windows)");
                CodePage.Add("1361");
                ShortNames.Add("Johab");
                LongNames.Add("Korean (Johab)");
                CodePage.Add("10000");
                ShortNames.Add("macintosh");
                LongNames.Add("Western European (Mac)");
                CodePage.Add("10001");
                ShortNames.Add("x-mac-japanese");
                LongNames.Add("Japanese (Mac)");
                CodePage.Add("10002");
                ShortNames.Add("x-mac-chineGetHTML()rad");
                LongNames.Add("Chinese Traditional (Mac)");
                CodePage.Add("10003");
                ShortNames.Add("x-mac-korean");
                LongNames.Add("Korean (Mac)");
                CodePage.Add("10004");
                ShortNames.Add("x-mac-arabic");
                LongNames.Add("Arabic (Mac)");
                CodePage.Add("10005");
                ShortNames.Add("x-mac-hebrew");
                LongNames.Add("Hebrew (Mac)");
                CodePage.Add("10006");
                ShortNames.Add("x-mac-greek");
                LongNames.Add("Greek (Mac)");
                CodePage.Add("10007");
                ShortNames.Add("x-mac-cyrillic");
                LongNames.Add("Cyrillic (Mac)");
                CodePage.Add("10008");
                ShortNames.Add("x-mac-chinesesimp");
                LongNames.Add("Chinese Simplified (Mac)");
                CodePage.Add("10010");
                ShortNames.Add("x-mac-romanian");
                LongNames.Add("Romanian (Mac)");
                CodePage.Add("10017");
                ShortNames.Add("x-mac-ukrainian");
                LongNames.Add("Ukrainian (Mac)");
                CodePage.Add("10021");
                ShortNames.Add("x-mac-thai");
                LongNames.Add("Thai (Mac)");
                CodePage.Add("10029");
                ShortNames.Add("x-mac-ce");
                LongNames.Add("Central European (Mac)");
                CodePage.Add("10079");
                ShortNames.Add("x-mac-icelandic");
                LongNames.Add("Icelandic (Mac)");
                CodePage.Add("10081");
                ShortNames.Add("x-mac-turkish");
                LongNames.Add("Turkish (Mac)");
                CodePage.Add("10082");
                ShortNames.Add("x-mac-croatian");
                LongNames.Add("Croatian (Mac)");
                CodePage.Add("20000");
                ShortNames.Add("x-Chinese-CNS");
                LongNames.Add("Chinese Traditional (CNS)");
                CodePage.Add("20001");
                ShortNames.Add("x-cp20001");
                LongNames.Add("TCA Taiwan");
                CodePage.Add("20002");
                ShortNames.Add("x-Chinese-Eten");
                LongNames.Add("Chinese Traditional (Eten)");
                CodePage.Add("20003");
                ShortNames.Add("x-cp20003");
                LongNames.Add("IBM5550 Taiwan");
                CodePage.Add("20004");
                ShortNames.Add("x-cp20004");
                LongNames.Add("TeleText Taiwan");
                CodePage.Add("20005");
                ShortNames.Add("x-cp20005");
                LongNames.Add("Wang Taiwan");
                CodePage.Add("20105");
                ShortNames.Add("x-IA5");
                LongNames.Add("Western European (IA5)");
                CodePage.Add("20106");
                ShortNames.Add("x-IA5-German");
                LongNames.Add("German (IA5)");
                CodePage.Add("20107");
                ShortNames.Add("x-IA5-Swedish");
                LongNames.Add("Swedish (IA5)");
                CodePage.Add("20108");
                ShortNames.Add("x-IA5-Norwegian");
                LongNames.Add("Norwegian (IA5)");
                CodePage.Add("20127");
                ShortNames.Add("us-ascii");
                LongNames.Add("US-ASCII");
                CodePage.Add("20261");
                ShortNames.Add("x-cp20261");
                LongNames.Add("T.61");
                CodePage.Add("20269");
                ShortNames.Add("x-cp20269");
                LongNames.Add("ISO-6937");
                CodePage.Add("20273");
                ShortNames.Add("IBM273");
                LongNames.Add("IBM EBCDIC (Germany)");
                CodePage.Add("20277");
                ShortNames.Add("IBM277");
                LongNames.Add("IBM EBCDIC (Denmark-Norway)");
                CodePage.Add("20278");
                ShortNames.Add("IBM278");
                LongNames.Add("IBM EBCDIC (Finland-Sweden)");
                CodePage.Add("20280");
                ShortNames.Add("IBM280");
                LongNames.Add("IBM EBCDIC (Italy)");
                CodePage.Add("20284");
                ShortNames.Add("IBM284");
                LongNames.Add("IBM EBCDIC (Spain)");
                CodePage.Add("20285");
                ShortNames.Add("IBM285");
                LongNames.Add("IBM EBCDIC (UK)");
                CodePage.Add("20290");
                ShortNames.Add("IBM290");
                LongNames.Add("IBM EBCDIC (Japanese katakana)");
                CodePage.Add("20297");
                ShortNames.Add("IBM297");
                LongNames.Add("IBM EBCDIC (France)");
                CodePage.Add("20420");
                ShortNames.Add("IBM420");
                LongNames.Add("IBM EBCDIC (Arabic)");
                CodePage.Add("20423");
                ShortNames.Add("IBM423");
                LongNames.Add("IBM EBCDIC (Greek)");
                CodePage.Add("20424");
                ShortNames.Add("IBM424");
                LongNames.Add("IBM EBCDIC (Hebrew)");
                CodePage.Add("20833");
                ShortNames.Add("x-EBCDIC-KoreanExtended");
                LongNames.Add("IBM EBCDIC (Korean Extended)");
                CodePage.Add("20838");
                ShortNames.Add("IBM-Thai");
                LongNames.Add("IBM EBCDIC (Thai)");
                CodePage.Add("20866");
                ShortNames.Add("koi8-r");
                LongNames.Add("Cyrillic (KOI8-R)");
                CodePage.Add("20871");
                ShortNames.Add("IBM871");
                LongNames.Add("IBM EBCDIC (Icelandic)");
                CodePage.Add("20880");
                ShortNames.Add("IBM880");
                LongNames.Add("IBM EBCDIC (Cyrillic Russian)");
                CodePage.Add("20905");
                ShortNames.Add("IBM905");
                LongNames.Add("IBM EBCDIC (Turkish)");
                CodePage.Add("20924");
                ShortNames.Add("IBM00924");
                LongNames.Add("IBM Latin-1");
                CodePage.Add("20932");
                ShortNames.Add("EUC-JP");
                LongNames.Add("Japanese (JIS 0208-1990 and 0212-1990)");
                CodePage.Add("20936");
                ShortNames.Add("x-cp20936");
                LongNames.Add("Chinese Simplified (GB2312-80)");
                CodePage.Add("20949");
                ShortNames.Add("x-cp20949");
                LongNames.Add("Korean Wansung");
                CodePage.Add("21025");
                ShortNames.Add("cp1025");
                LongNames.Add("IBM EBCDIC (Cyrillic Serbian-Bulgarian)");
                CodePage.Add("21866");
                ShortNames.Add("koi8-u");
                LongNames.Add("Cyrillic (KOI8-U)");
                CodePage.Add("28591");
                ShortNames.Add("iso-8859-1");
                LongNames.Add("Western European (ISO)");
                CodePage.Add("28592");
                ShortNames.Add("iso-8859-2");
                LongNames.Add("Central European (ISO)");
                CodePage.Add("28593");
                ShortNames.Add("iso-8859-3");
                LongNames.Add("Latin 3 (ISO)");
                CodePage.Add("28594");
                ShortNames.Add("iso-8859-4");
                LongNames.Add("Baltic (ISO)");
                CodePage.Add("28595");
                ShortNames.Add("iso-8859-5");
                LongNames.Add("Cyrillic (ISO)");
                CodePage.Add("28596");
                ShortNames.Add("iso-8859-6");
                LongNames.Add("Arabic (ISO)");
                CodePage.Add("28597");
                ShortNames.Add("iso-8859-7");
                LongNames.Add("Greek (ISO)");
                CodePage.Add("28598");
                ShortNames.Add("iso-8859-8");
                LongNames.Add("Hebrew (ISO-Visual)");
                CodePage.Add("28599");
                ShortNames.Add("iso-8859-9");
                LongNames.Add("Turkish (ISO)");
                CodePage.Add("28603");
                ShortNames.Add("iso-8859-13");
                LongNames.Add("Estonian (ISO)");
                CodePage.Add("28605");
                ShortNames.Add("iso-8859-15");
                LongNames.Add("Latin 9 (ISO)");
                CodePage.Add("29001");
                ShortNames.Add("x-Europa");
                LongNames.Add("Europa");
                CodePage.Add("38598");
                ShortNames.Add("iso-8859-8-i");
                LongNames.Add("Hebrew (ISO-Logical)");
                CodePage.Add("50220");
                ShortNames.Add("iso-2022-jp");
                LongNames.Add("Japanese (JIS)");
                CodePage.Add("50221");
                ShortNames.Add("csISO2022JP");
                LongNames.Add("Japanese (JIS-Allow 1 byte Kana)");
                CodePage.Add("50222");
                ShortNames.Add("iso-2022-jp");
                LongNames.Add("Japanese (JIS-Allow 1 byte Kana - SO/SI)");
                CodePage.Add("50225");
                ShortNames.Add("iso-2022-kr");
                LongNames.Add("Korean (ISO)");
                CodePage.Add("50227");
                ShortNames.Add("x-cp50227");
                LongNames.Add("Chinese Simplified (ISO-2022)");
                CodePage.Add("51932");
                ShortNames.Add("euc-jp");
                LongNames.Add("Japanese (EUC)");
                CodePage.Add("51936");
                ShortNames.Add("EUC-CN");
                LongNames.Add("Chinese Simplified (EUC)");
                CodePage.Add("51949");
                ShortNames.Add("euc-kr");
                LongNames.Add("Korean (EUC)");
                CodePage.Add("52936");
                ShortNames.Add("hz-gb-2312");
                LongNames.Add("Chinese Simplified (HZ)");
                CodePage.Add("54936");
                ShortNames.Add("GB18030");
                LongNames.Add("Chinese Simplified (GB18030)");
                CodePage.Add("57002");
                ShortNames.Add("x-iscii-de");
                LongNames.Add("ISCII Devanagari");
                CodePage.Add("57003");
                ShortNames.Add("x-iscii-be");
                LongNames.Add("ISCII Bengali");
                CodePage.Add("57004");
                ShortNames.Add("x-iscii-ta");
                LongNames.Add("ISCII Tamil");
                CodePage.Add("57005");
                ShortNames.Add("x-iscii-te");
                LongNames.Add("ISCII Telugu");
                CodePage.Add("57006");
                ShortNames.Add("x-iscii-as");
                LongNames.Add("ISCII Assamese");
                CodePage.Add("57007");
                ShortNames.Add("x-iscii-or");
                LongNames.Add("ISCII Oriya");
                CodePage.Add("57008");
                ShortNames.Add("x-iscii-ka");
                LongNames.Add("ISCII Kannada");
                CodePage.Add("57009");
                ShortNames.Add("x-iscii-ma");
                LongNames.Add("ISCII Malayalam");
                CodePage.Add("57010");
                ShortNames.Add("x-iscii-gu");
                LongNames.Add("ISCII Gujarati");
                CodePage.Add("57011");
                ShortNames.Add("x-iscii-pa");
                LongNames.Add("ISCII Punjabi");
                CodePage.Add("65000");
                ShortNames.Add("utf-7");
                LongNames.Add("Unicode (UTF-7)");
                CodePage.Add("65001");
                ShortNames.Add("utf-8");
                LongNames.Add("Unicode (UTF-8)");
                CodePage.Add("65005");
                ShortNames.Add("utf-32");
                LongNames.Add("Unicode (UTF-32)");
                CodePage.Add("65006");
                ShortNames.Add("utf-32BE");
                LongNames.Add("Unicode (UTF-32 Big-Endian)");

                #endregion
            }

            switch (listType)
            {
                case CharSetInfo.CODEPAGE:
                    return CodePage;
                case CharSetInfo.SHORTNAMES:
                    return ShortNames;
                case CharSetInfo.LONGNAMES:
                    return LongNames;
            }
            return ShortNames;
        }
    }
}
