﻿using System;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {
        public static DateTime DecodeDateTime(string input, out TimeSpan timezone)
        {
            int year = 2000 + GetIntFromExchangedByte(input[0]);
            int month = GetIntFromExchangedByte(input[1]);
            int day = GetIntFromExchangedByte(input[2]);
            int hour = GetIntFromExchangedByte(input[3]);
            int min = GetIntFromExchangedByte(input[4]);
            int sec = GetIntFromExchangedByte(input[5]);
            int tz = GetIntFromExchangedByte(input[6]);

            //Relation to GMT. One unit is 15 min. If MSB=1, value is negative.
            int elojel = 1;
            if (tz > 0x7F)
            {
                elojel = -1;
                tz = tz & 0x7F;
            }

            timezone = new TimeSpan(0, elojel * (tz * 15), 0);
            try
            {
                var dateTime = new DateTime(year, month, day, hour, min, sec, DateTimeKind.Local);
                return dateTime.AddMinutes((OzDateTime.Now - OzDateTime.UtcNow).TotalMinutes - (elojel * (tz * 15)));
            }
            catch
            {
                return OzDateTime.Now;
            }
        }

        public static DateTime DecodeDateTime(string input, string dateTimeFormat = "")
        {
            //https://www.c-sharpcorner.com/blogs/date-and-time-format-in-c-sharp-programming1
            var decoded = DateTime.MinValue;
            var dateTimeFormats = new string[]
            {
                dateTimeFormat,
                "yyMMddHHmm",
                "yyMMddHHmmss",
                "dd-MMM-yyHH:mm",
                "dd-MMM-yyHH:mm:ss",
                "dd-MMM-yy HH:mm",
                "dd-MMM-yy HH:mm:ss",
                "yyyyMMddHHmmss",
                "yyyyMMddHHmm",
            };

            for (var i = 0; i < dateTimeFormats.Length; i++)
            {
                try
                {
                    string format = dateTimeFormats[i];
                    if (string.IsNullOrEmpty(format)) continue;
                    decoded = DateTime.ParseExact(input, format, System.Globalization.CultureInfo.InvariantCulture);
                    return decoded;
                }
                catch { }
            }

            return decoded = DateTime.Now;
        }

        public static string TimespanToString(TimeSpan timespan)
        {
            string tss;
            var ts = timespan.TotalSeconds;
            if (ts < 60)
            {
                tss = String.Format("{0:0.00}", ts) + " seconds";
            }
            else if (ts < 3600)
            {
                var mins = ts / 60;
                tss = String.Format("{0:0.00}", mins) + " minutes";
            }
            else
            {
                var hrs = ts / 3600;
                tss = String.Format("{0:0.00}", hrs) + " hours";
            }
            return tss;
        }
    }
}
