﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {
        public enum SizeUnit
        {
            Decimal = 1000,
            Binary = 1024
        }

        public static string ReadableFileSize(double value, SizeUnit unit = SizeUnit.Binary)
        {
            int deep = 0;
            return ConvertSize(value, unit, ref deep);
        }

        static string ConvertSize(double value, SizeUnit unit, ref int deep)
        {
            var divisor = (double)unit;
            var converted = value / divisor;
            if (converted > divisor)
            {
                deep++;
                return ConvertSize(converted, unit, ref deep);
            }
            converted = Math.Round(converted, 2);
            switch (deep)
            {
                case 0: return $"{converted} KB";
                case 1: return $"{converted} MB";
                case 2: return $"{converted} GB";
                case 3: return $"{converted} TB";
                case 4: return $"{converted} PB";
                case 5: return $"{converted} EB";
            }
            return converted.ToString();
        }

        public static string MakeValidFileName(string filename)
        {
            filename = filename.Trim(Path.GetInvalidFileNameChars());
            filename = filename.Trim(Path.GetInvalidPathChars());
            string[] split = filename.Split(' ', '_');
            string validFileName = string.Empty;
            foreach (var item in split)
            {
                validFileName += UpperFirst(item);
            }
            return validFileName;
        }
    }
}
