﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Globalization;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {
        public static bool HTMLTagsPresent(string s)
        {
            if (s.Contains("<") && s.Contains(">")) return true;
            return false;
        }

        public static string HTMLRemoveTags(string s)
        {
            if (String.IsNullOrEmpty(s)) return "";
            return Regex.Replace(s, "<[^>]*>", "");
        }

        public static string HtmlRemoveTagsAndUnicode(string s)
        {
            if (String.IsNullOrEmpty(s)) return "";
            s = Regex.Replace(s, "<[^>]*>", " ");
            s = Regex.Replace(s, "\\s\\s*", " ");
            s = HtmlDecodeMono(s);
            s = removeUnicodeEmojies(s);
            s = s.Trim();
            return s;
        }

        public static string removeUnicodeEmojies(string text)
        {
            var reg = new Regex(@"[^\u0000-\u01FF]", RegexOptions.Compiled);
            var ret = reg.Replace(text, string.Empty);
            var regUniNbsp = new Regex(@"[\u00A0]", RegexOptions.Compiled);
            ret = regUniNbsp.Replace(ret, " ");
            return ret;
        }

        public static string HTMLToPlainText(string s)
        {
            if (String.IsNullOrEmpty(s)) return "";
            s = HTMLRemoveTags(s);
            s = OzHttpUtility.HtmlDecode(s);
            return Regex.Replace(s, @"\s{2,}", " ");
        }

        

        public static string MinifyHTML(string markup)
        {
            markup = RemoveHtmlComments(markup);
            markup = RemoveLineBreaks(markup);
            markup = RemoveWhiteSpaces(markup);
            markup = RemoveWhitespaceFromHtmlPage(markup);
            return markup;
        }

        private static readonly Regex RegexBetweenTags = new Regex(@"&gt;(?! )\s+", RegexOptions.Compiled);

        public static string RemoveWhitespaceFromHtmlPage(string html)
        {
            html = RegexBetweenTags.Replace(html, "&gt;");
            return html.Trim();
        }

        public static string RemoveApostrofe(string markup)
        {
            return Regex.Replace(markup, "\"", string.Empty, RegexOptions.Compiled | RegexOptions.Multiline);
        }

        public static string RemoveHtmlComments(string markup)
        {
            return Regex.Replace(markup, "<!--*.*?-->", string.Empty, RegexOptions.Compiled | RegexOptions.Multiline);
        }

        public static string RemoveLineBreaks(string str)
        {
            str = Regex.Replace(str, "\r", string.Empty, RegexOptions.Compiled | RegexOptions.Multiline);
            var ret = Regex.Replace(str, "\n", string.Empty, RegexOptions.Compiled | RegexOptions.Multiline);
            return ret;
        }

        public static string RemoveWhiteSpaces(string markup)
        {
            markup = Regex.Replace(markup, "^\\s*", string.Empty, RegexOptions.Compiled | RegexOptions.Multiline);
            markup = Regex.Replace(markup, @"\s*(<[^>]+>)\s*", "$1", RegexOptions.Singleline);
            return markup;
        }

        public static string RemoveWhitespace(this string str)
        {
            return string.Join("", str.Split(default(string[]), StringSplitOptions.RemoveEmptyEntries));
        }

        public static string HtmlSpecialChars(string text,bool convertWhiteSpace=false)
        {
            text = OzHttpUtility.HtmlEncode(text);
            if (convertWhiteSpace)
            {
                text = text.Replace("\r\n", "\r");
                text = text.Replace("\n", "\r");
                text = text.Replace("\r", "<br>\r\n");
                text = text.Replace("  ", " &nbsp;");
            }
            return text;
        }

        public static string HtmlTagSafe(string message)
        {
            return OzHttpEncoder.HtmlEncode(message);
        }

        /// <summary>
        /// Wordlist is a semicolon separated list of words
        /// </summary>
        /// <param name="wordlist"></param>
        /// <param name="content"></param>
        /// <returns></returns>
        public static bool ContainsAnyOfTheWords(string wordlist, string content)
        {
            if (string.IsNullOrWhiteSpace(wordlist)) return false;
            if (string.IsNullOrEmpty(wordlist.Trim())) return false;

            var words = wordlist.Split(';');
            foreach (var word in words)
            {
                var wordTrimmed = word.Trim();
                if (string.IsNullOrEmpty(wordTrimmed)) continue;
                var re = new Regex("\\W" + wordTrimmed + "\\W", RegexOptions.Multiline | RegexOptions.IgnoreCase | RegexOptions.CultureInvariant);
                if (re.Match(" " + content + " ").Success)
                    return true;
            }
            return false;
        }
    }
}
