﻿using System;
using System.Globalization;
using System.Text;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {
        public static string FromHexString(string s)
        {
            s = s.ToUpper(CultureInfo.InvariantCulture);
            var x = 0;

            var l = (s.Length /2) * 2;
            var tst = new StringBuilder(string.Empty);
            while (x < l)
            {
                if (x + 1 < l)
                    tst.Append((char)(FromHexChar(s[x]) * 16 + FromHexChar(s[x + 1])));
                else
                    tst.Append((char)(FromHexChar(s[x])));
                x = x + 2;
            }
            return tst.ToString();
        }

        public static char ToHexChar(int i)
        {
            return i < 10 ? (char)(('0') + i) : (char)(('A') + i - 10);
        }

        public static int FromHexChar(char i)
        {
            return i < 'A' ? (i) - '0' : (i) - 'A' + 10;
        }

        public static string ToHyphenedHexString(byte[] bytes)
        {
            return BitConverter.ToString(bytes);
        }

        public static byte[] FromHexStringFast(string hexString)
        {
            try
            {
                var cleanHexString = hexString.Replace("-", "");
                var numberChars = cleanHexString.Length;
                var bytes = new byte[numberChars / 2];
                for (var i = 0; i < numberChars; i += 2)
                {
                    bytes[i / 2] = Convert.ToByte(cleanHexString.Substring(i, 2), 16);
                }
                return bytes;
            }
            catch (Exception)
            {
                return new byte[0];
            }
        }

        public static byte[] FromHexStringSafe(string input)
        {
            var value = new Byte[input.Length / 2];
            for (var index = 0; index < value.Length; index++)
            {
                Byte.TryParse(input.Substring(index * 2, 2), NumberStyles.HexNumber, null,
                              out value[index]);
            }
            return value;
        }

        public static string ToHexString(string s)
        {
            var hexStr = new StringBuilder();
            foreach (var c in s)
            {
                hexStr.Append(string.Format("{0:x2}", (int)c));
            }
            return hexStr.ToString().ToUpper();
        }

        public static string ToWideHexString(string s)
        {
            var l = s.Length;
            var tst = new StringBuilder();
            for (var x = 0; x < l; x++)
            {
                var s1 = (char)(s[x] / 256);
                var s2 = (char)(s[x] % 256);

                var c = ToHexChar((s1) / 16);
                tst.Append(c);
                c = ToHexChar((s1) % 16);
                tst.Append(c);

                c = ToHexChar((s2) / 16);
                tst.Append(c);
                c = ToHexChar((s2) % 16);
                tst.Append(c);
            }
            return tst.ToString().ToUpper();
        }

        public static bool IsHexChar(char c)
        {
            return ((c >= '0') && (c <= '9')) || ((c >= 'A') && (c <= 'F'));
        }

        public static bool IsHexString(string s)
        {
            if (string.IsNullOrEmpty(s)) return false;
            for (int x = 0; x < s.Length; x++)
                if (!IsHexChar(s[x])) return false;
            return true;
        }

        public static int FromHexByte(string s)
        {
            if (s.Length != 2)
            {
                return 0;
            }
            var b1 = s[0];
            var b2 = s[1];
            return IsHexChar(b1) && IsHexChar(b2) ? FromHexChar(b1) * 16 + FromHexChar(b2) : 0;
        }

        public static string OzIntToHex(long i)
        {
            var ret = string.Empty;
            if (i == 0) ret = "0";
            while (i > 0)
            {
                ret = ToHexChar((int)(i % 16)) + ret;
                i = i / 16;
            }
            return ret;
        }

        public static byte[] OzUshortToHexBytes(ushort i)
        {
            byte[] intBytes = BitConverter.GetBytes(i);
            if (BitConverter.IsLittleEndian)
                Array.Reverse(intBytes);
            byte[] result = intBytes;
            return result;
        }

        public static byte[] OzIntToHexBytes(uint i)
        {
            byte[] intBytes = BitConverter.GetBytes(i);
            if (BitConverter.IsLittleEndian)
                Array.Reverse(intBytes);
            byte[] result = intBytes;
            return result;
        }

        public static byte[] OzIntToHexBytes(int i)
        {
            byte[] intBytes = BitConverter.GetBytes(i);
            if (BitConverter.IsLittleEndian)
                Array.Reverse(intBytes);
            byte[] result = intBytes;
            return result;
        }

        public static string OzIntToHexByte(int i)
        {
            return OzIntToHex(i).PadLeft(2, '0');
        }

        public static string ToHexString(char c)
        {
            return OzIntToHex((int)c).PadLeft(2, '0');
        }

        public static string ToHexString(byte b)
        {
            var ret = new StringBuilder();
            ret.AppendFormat("{0:x2}", b);
            return ret.ToString();
        }

        public const string HexAlphabet = "0123456789ABCDEF";
        public static string ToHexString(byte[] bytes)
        {
            if (bytes == null) return "";
            var ret = new StringBuilder();
            foreach (byte B in bytes)
            {
                ret.Append(HexAlphabet[(int)(B >> 4)]);
                ret.Append(HexAlphabet[(int)(B & 0xF)]);
            }
            return ret.ToString();
        }

        public static string ToBinString(byte b)
        {
            string binaryString = Convert.ToString(b, 2).PadLeft(8, '0');
            return binaryString;
        }


        public static byte[] FromHexStringToBytes(string s)
        {
            s = s.ToUpper(CultureInfo.InvariantCulture);
            var x = 0;
            var y = 0;
            var l = (s.Length / 2)*2;
            var b = new byte[s.Length / 2];
            while (x < l)
            {
                b[y++] = (byte)(FromHexChar(s[x]) * 16 + FromHexChar(s[x + 1]));
                x = x + 2;
            }
            return b;
        }

        public static bool IsHexDigit(char c)
        {
            return ((c >= '0') && (c <= '9')) || ((c >= 'A') && (c <= 'F'));
        }

        public static byte[] ConvertHexStringToBytes(string hex, char separator = '-')
        {
            var hexVals = hex.Split(separator);
            var ret = new byte[hexVals.Length];
            for (int i = 0; i < hexVals.Length; i++)
            {
                byte b = Convert.ToByte(hexVals[i], 16);
                ret[i] = b;
            }
            return ret;
        }
    }
}
