﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {
        public static bool IsNumber(string numstr)
        {
            if (numstr.Length == 0)
            {
                return false;
            }

            for (var i = 0; i < numstr.Length; i++)
            {
                if ((numstr[i] < '0') || (numstr[i] > '9')) return false;
            }
            return true;
        }


        public static int String2Int(string s)
        {
            int x;
            return int.TryParse(s, out x) ? x : 0;
        }

        public static int ToInt32(byte[] value, int startIndex = 0)
        {
            int intValue = BitConverter.ToInt32(value, startIndex);
            if (BitConverter.IsLittleEndian)
                intValue = BitConverter.ToInt32(value.Reverse().ToArray(), value.Length - sizeof(Int32) - startIndex);

            return intValue;
        }

        public static uint ToUInt32(byte[] value, int startIndex = 0)
        {
            uint uIntValue = BitConverter.ToUInt32(value, startIndex);
            if (BitConverter.IsLittleEndian)
                uIntValue = BitConverter.ToUInt32(value.Reverse().ToArray(), value.Length - sizeof(UInt32) - startIndex);

            return uIntValue;
        }

        public static int ToIntNumber(string s)
        {
            if (IsNumber(s))
            {
                return int.Parse(s);
            }
            else
            {
                return 0;
            }
        }

        public static bool IsEven(int x)
        {
            return (x % 2) != 1;
        }

        public static bool IsOdd(int x)
        {
            return !IsEven(x);
        }

        public static string UIntVal(int i)
        {
            string s;
            var last7 = i & 0x7F;

            if (i == last7)
            {
                s = ((char)last7).ToString();
            }
            else
            {
                s = ((char)last7).ToString();
                while (i > last7)
                {
                    i = i >> 7;
                    last7 = i & 0x7F;
                    s = ((char)(last7 + 0x80)) + s;
                }
            }
            return s;
        }

        public static string LittleEndianEncode(int i)
        {
            var s = new StringBuilder();
            var last7 = i & 0x7F;
            s.Append((char)last7);
            while (i != last7)
            {
                i = i >> 7;
                last7 = i & 0x7F;
                s.Insert(0, (char)(last7 | 0x80));
            }
            return s.ToString();
        }

        public static int CombineHashCodes(IEnumerable<int> hashCodes)
        {
            int hash1 = (5381 << 16) + 5381;
            int hash2 = hash1;

            int i = 0;
            foreach (var hashCode in hashCodes)
            {
                if (i % 2 == 0)
                    hash1 = ((hash1 << 5) + hash1 + (hash1 >> 27)) ^ hashCode;
                else
                    hash2 = ((hash2 << 5) + hash2 + (hash2 >> 27)) ^ hashCode;

                ++i;
            }

            return hash1 + (hash2 * 1566083941);
        }

        public static int ConvertToInt(string param, int def)
        {
            try
            {
                return Convert.ToInt32(param);
            }
            catch
            {
                return def;
            }
        }

        public static double ToDoubleInvariant(string s)
        {
            double value = 0;

            if (string.IsNullOrEmpty(s)) return value;

            double.TryParse(s.Replace(',', '.'), NumberStyles.Number, CultureInfo.InvariantCulture, out value);
            return value;
        }

        public static bool TryToDoubleInvariant(string s, out double d)
        {
            if (string.IsNullOrEmpty(s))
            {
                d = 0;
                return false;
            }

            return double.TryParse(s.Replace(',', '.'), NumberStyles.Number, CultureInfo.InvariantCulture, out d);
        }

        public static string FromDoubleInvariant(double d)
        {
            return d.ToString(CultureInfo.InvariantCulture);
        }

        public static float Remap(float s, float a1, float a2, float b1, float b2)
        {
            return b1 + (s - a1) * (b2 - b1) / (a2 - a1);
        }

        public static double Remap(double s, double a1, double a2, double b1, double b2)
        {
            return b1 + (s - a1) * (b2 - b1) / (a2 - a1);
        }
    }
}
