﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {
        public static bool IsValidHostnameOrIP(string hostnameOrIp)
        {
            return IsValidHostname(hostnameOrIp) || IsValidIP(hostnameOrIp);
        }

        public static bool IsValidHostname(string hostname)
        {

            var validHostnameRegex = @"^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])$";
            return Regex.Match(hostname, validHostnameRegex).Success;
        }

        public static bool IsValidIP(string ip)
        {
            var validIpAddressRegex = @"^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$";
            return Regex.Match(ip, validIpAddressRegex).Success;
        }

        public static bool IsLocalIP(string ip)
        {
            if (!IsValidIP(ip)) return false;

            if (ip.StartsWith("192.168.")) return true;
            if (ip.StartsWith("10.")) return true;
            if (ip.StartsWith("172."))
            {
                var ipparts = ip.Split('.');
                if (!int.TryParse(ipparts[1], out var second)) return false;
                if (second >= 16 && second <= 31) return true;
            }
            return false;
        }

        public static string GetStation(string username_at_domain_at_station)
        {
            GetUsernameAndDomainAndStation(username_at_domain_at_station, out string username, out string domain, out string station);
            return station;
        }

        public static string GetStationShort(string fullStationID)
        {
            if (string.IsNullOrEmpty(fullStationID)) return "";
            return fullStationID.Substring(8) + "...";
        }

        public static string GetDomain(string username_at_domain)
        {
            GetUsernameAndDomainAndStation(username_at_domain, out string username, out string domain, out string station);
            return domain;
        }

        public static string GetUsername(string username_at_domain)
        {
            GetUsernameAndDomainAndStation(username_at_domain, out string username, out string domain, out string station);
            return username;
        }

        public static string simplifytoUsernameAndDomain(string username_at_domain_at_station)
        {
            GetUsernameAndDomainAndStation(username_at_domain_at_station, out string username, out string domain, out string station);
            return username + "@" + domain;
        }

        public static void GetUsernameAndDomain(string username_at_domain, out string username, out string domain)
        {
            GetUsernameAndDomainAndStation(username_at_domain, out username, out domain, out string station);
        }

        public static string GetUsernameAndDomainWithoutStation(string username_at_domain_at_station)
        {
            GetUsernameAndDomainAndStation(username_at_domain_at_station, out string username, out string domain, out string station);
            return username + "@" + domain;
        }

        public static string changeLocalhostToDomain(string username_at_domain, string domain, bool matchAtSign = true)
        {
            GetUsernameAndDomainAndStation(username_at_domain, out string username, out string dom, out string station);
            if (domain.ToLowerInvariant() != "localhost") return username_at_domain;
            return username + "@" + domain + (string.IsNullOrEmpty(station) ? "" : "@" + station);
        }

        public static string changedomainToLocalhost(string username_at_domain)
        {
            GetUsernameAndDomainAndStation(username_at_domain, out string username, out string domain, out string station);
            return username + "@localhost" + (string.IsNullOrEmpty(station) ? "" : "@" + station);
        }

        public static string removeDomainLocalhost(string username_at_domain)
        {
            if (username_at_domain.EndsWith("@localhost")) return username_at_domain.Replace("@localhost", "");
            return username_at_domain;
        }

        public static void GetTypeNameAndNicenameAndUsernameAndDomainAndStationAndAddress(string s, out string typename, out string typenamenice, out string fullname, out string user, out string domain, out string station, out string address, out bool addressRequired)
        {
            typename = "";
            typenamenice = "";
            GetNicenameAndUsernameAndDomainAndStationAndAddress(s, out fullname, out user, out domain, out station, out address, out addressRequired);
            if (string.IsNullOrEmpty(fullname)) return;
            if (!fullname.Contains(" <")) return;
            if (!fullname.Contains(">")) return;
            var p1 = fullname.IndexOf(" <");
            var p2 = fullname.IndexOf(">");
            if (p2 < p1) return;
            var typeinfo = fullname.Substring(p1 + 2, p2 - p1 - 2);
            fullname = fullname.Substring(0, p1);
            if (typeinfo.Length < 1) return;
            if (!typeinfo.Contains(",")) return;
            var parts = typeinfo.Split(new char[] { ',' });
            typename = parts[0];
            typenamenice = parts[1];
        }

        public static void GetNicenameAndUsernameAndDomainAndStationAndAddress(string nicename_plus_username_at_domain_at_station_with_address, out string nicename, out string user, out string domain, out string station, out string address, out bool addressRequired)
        {
            nicename = "";
            user = "";
            domain = "";
            station = "";
            address = "";
            addressRequired = false;
            var s = nicename_plus_username_at_domain_at_station_with_address.Trim();
            if (!s.StartsWith("\""))
            {
                GetUsernameAndDomainAndStationAndAddress(s, out user, out domain, out station, out address, out addressRequired);
                return;
            }
            var posOfClosingQuote = s.Substring(1).IndexOf('"');
            if (posOfClosingQuote == -1) return;
            nicename = s.Substring(1, posOfClosingQuote);
            var tail = s.Substring(posOfClosingQuote + 3);
            GetUsernameAndDomainAndStationAndAddress(tail, out user, out domain, out station, out address, out addressRequired);
        }

        public static string buildUsernameAndDomainAndStationAndAddress(string user, string domain, string station = null, string address = null, bool addressRequired = false)
        {
            var ret = new StringBuilder();
            ret.Append(user + "@" + domain);
            if (!string.IsNullOrEmpty(station))
                ret.Append("@" + station);
            if (string.IsNullOrEmpty(address)) return ret.ToString();

            if (addressRequired)
                ret.Append(" [" + address + "]");
            else
                ret.Append(" (" + address + ")");
            return ret.ToString();
        }

        public static void GetUsernameAndDomainAndStationAndAddress(string username_at_domain_at_station_with_address, out string user, out string domain, out string station, out string address, out bool addressRequired)
        {
            user = "";
            domain = "";
            station = "";
            address = "";
            addressRequired = false;

            var s = username_at_domain_at_station_with_address.Trim();

            if (!s.Contains(" "))
            {
                GetUsernameAndDomainAndStation(s, out user, out domain, out station);
                return;
            }
            var parts = s.Split(new char[] { ' ' });
            GetUsernameAndDomainAndStation(parts[0], out user, out domain, out station);
            if (parts[1].Length < 1) return;
            if (parts[1].Substring(0, 1) == "[") addressRequired = true;
            if (parts[1].Length < 3) return;
            address = parts[1].Substring(1, parts[1].Length - 2);
        }

        public static void GetConnectionIDAndStation(string username_at_domain_at_station, out string connectionID, out string station)
        {
            GetUsernameAndDomainAndStation(username_at_domain_at_station, out var un, out var dom, out station);
            connectionID = un + "@" + dom;
        }

        public static void GetUsernameAndDomainAndStation(string username_at_domain_at_station, out string username, out string domain, out string station)
        {
            username = "";
            domain = "";
            station = "";
            if (string.IsNullOrEmpty(username_at_domain_at_station)) return;

            if (!username_at_domain_at_station.Contains("@"))
            {
                username = username_at_domain_at_station;
                return;
            }

            var parts = username_at_domain_at_station.Split('@');

            //Station
            if (parts.Length > 2)
            {
                station = parts[parts.Length - 1];
            }

            //Domain
            if (parts.Length == 2)
            {
                domain = parts[1];
            }
            else if (parts.Length > 2)
            {
                domain = parts[parts.Length - 2];
            }

            //Username
            if (parts.Length == 2)
            {
                username = parts[0];
                return;
            }

            for (int x = 0; x < parts.Length - 2; x++)
            {
                username += parts[x];
                if (parts.Length - 2 > x + 1) username += "@";
            }
        }

       
    }
}

