﻿using System;
using System.Collections.Generic;
using System.Globalization;

namespace Ozeki
{
    partial class OzCommonStrings
    {
        public static bool TryParseToDictionary(string[] lines, string delimiter, out Dictionary<string, string> dictionary)
        {
            dictionary = new Dictionary<string, string>();
            foreach (var field in lines)
            {
                var kvPair = field.Split(new string[] { delimiter }, StringSplitOptions.None);
                if (kvPair.Length != 2)
                    continue;
                if (dictionary.ContainsKey(kvPair[0]))
                    return false;
                dictionary.Add(kvPair[0], kvPair[1]);
            }
            return true;
        }

        private static readonly NumberFormatInfo _pointCommaStyle = new NumberFormatInfo() { NumberDecimalSeparator = ".", NumberGroupSeparator = "," };
        private static readonly NumberFormatInfo _commaPointStyle = new NumberFormatInfo() { NumberDecimalSeparator = ",", NumberGroupSeparator = "." };

        public static double ParseDouble(string s, double defaultVal)
        {
            if (string.IsNullOrEmpty(s))
                return defaultVal;

            double res = defaultVal;

            if(s.Contains(_pointCommaStyle.NumberDecimalSeparator) && s.Contains(_pointCommaStyle.NumberGroupSeparator))
            {
                if (double.TryParse(s, NumberStyles.Float | NumberStyles.AllowThousands, _pointCommaStyle, out res))
                    return res;
                if (double.TryParse(s, NumberStyles.Float | NumberStyles.AllowThousands, _commaPointStyle, out res))
                    return res;
            }

            bool formatError = false;
            try
            {
                res = double.Parse(s.Replace(',', '.'), CultureInfo.InvariantCulture);
            }
            catch(FormatException ex)
            {
                formatError = true;
            }

            if(formatError)
            {
                formatError = false;
                try
                {
                    res = double.Parse(s.Replace('.', ','), NumberStyles.Float | NumberStyles.AllowThousands, CultureInfo.InvariantCulture);
                }
                catch (FormatException ex)
                {
                    formatError = true;
                }
            }

            return res;
        }
    }
}
