﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Globalization;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;

using System.Web;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {
        public static string SimplifyPath(string path)
        {
            var retpath = String.Empty;
            var backpos = 0;
            var pathelements = path.Split(Path.DirectorySeparatorChar);
            for (var x = pathelements.Length - 1; x >= 0; x--)
            {
                if (pathelements[x] == "..")
                {
                    backpos++;
                }
                else if (backpos > 0)
                {
                    backpos--;
                }
                else
                {
                    if (retpath.Length > 0)
                    {
                        retpath = pathelements[x] + Path.DirectorySeparatorChar + retpath;
                    }
                    else
                    {
                        retpath = pathelements[x];
                    }
                }
            }
            return retpath;
        }

        public static string ToFileName(string fn)
        {
            return ToFileName(fn, false);
        }

        public static string ToFileName(string fn, bool allowDots)
        {
            return ToFileName(fn, allowDots, true);
        }

        public static string ToFileName(string fn, bool allowDots, bool combineUnderscore)
        {
            if (String.IsNullOrEmpty(fn)) return "";
            var matchPattern = "[^a-zA-Z0-9_]";
            if (allowDots) matchPattern = "[^a-zA-Z0-9_.]";

            var ret = Regex.Replace(fn, matchPattern, "_", RegexOptions.IgnoreCase);
            if (combineUnderscore) ret= Regex.Replace(ret, "__+", "_", RegexOptions.IgnoreCase);

            return ret;
        }

        public static string ToLettersAndNumbersOnly(string s)
        {
            return ToFileName(s, false);
        }

            public static string ExtractFileName(string fullpath)
        {
            var lp = fullpath.LastIndexOf(Path.DirectorySeparatorChar);
            var filename = lp > -1 ? fullpath.Substring(lp + 1, fullpath.Length - lp - 1) : fullpath;
            return filename;
        }

        public static string PreparePath(string path)
        {
            if (string.IsNullOrEmpty(path)) return path;
            string ret = path.Replace(@"/", Path.DirectorySeparatorChar.ToString()).Replace(@"\", Path.DirectorySeparatorChar.ToString());
            return ret;
        }

        public static string PathCombine(params string[] pathSegments)
        {
            var ret = new StringBuilder();
            bool first = true;
            foreach (var pathSegment in pathSegments)
            {
                if (first)
                {
                    ret.Append(prepareForPathCombine(pathSegment));
                }
                else
                {
                    ret.Append(FileSepChar);
                    var s = prepareForPathCombine(pathSegment);
                    while (s.StartsWith(FileSepChar.ToString())) s = s.Substring(1);
                    ret.Append(s);
                }
                first = false;
            }
            return ret.ToString();
        }

        static string prepareForPathCombine(string pathPart)
        {
            var ret = pathPart.Replace('/', FileSepChar).Replace('\\', FileSepChar);
            if (ret.EndsWith(FileSepChar.ToString())) ret = ret.Substring(0, ret.Length - 1);
            return ret;
        }

        public static string PathChoot(string root, string inputPath)
        {
            inputPath = inputPath.Replace("..\\","");
            inputPath = inputPath.Replace("..//", "");
            inputPath = inputPath.Replace("%", "");
            inputPath = inputPath.Replace("//", "_OZ_SEP_");
            inputPath = inputPath.Replace("\\", "_OZ_SEP_");

            string regexSearch = new string(Path.GetInvalidFileNameChars()) + new string(Path.GetInvalidPathChars());
            Regex r = new Regex(string.Format("[{0}]", Regex.Escape(regexSearch)));
            inputPath = r.Replace(inputPath, "");

            inputPath = inputPath.Replace("_OZ_SEP_",Path.DirectorySeparatorChar.ToString());
            var ret = Path.Combine(root, inputPath);
            return ret;
        }

        public static string GetLatestVersionPath(string basepath)
        {
            try
            {
                if (!Directory.Exists(basepath)) return "v1.0.0";
                var subdirectories = Directory.GetDirectories(basepath);

                Version latestVersion = null;

                foreach (var subdirectory in subdirectories)
                {
                    string versionString = Path.GetFileName(subdirectory);
                    if (versionString.StartsWith("v")) versionString = versionString.Substring(1);

                    // Parse the version string
                    if (Version.TryParse(versionString, out var version))
                    {
                        if (latestVersion == null || version > latestVersion)
                        {
                            latestVersion = version;
                        }
                    }
                }

                if (latestVersion == null) return "v1.0.0";
                return latestVersion.ToString();
            } catch
            {
                return "v1.0.0";
            }

        }

    }
}

