﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {

        public static bool WordChar(char c)
        {
            return ((c >= 'a') && (c <= 'z')) || ((c >= 'A') && (c <= 'Z')) || ((c >= '0') && (c <= '9'));
        }

        public static bool IsEmail(string email)
        {
            if (string.IsNullOrEmpty(email)) return false;

            const string strRegex = @"^([a-zA-Z0-9_\-\.\+]+)@((\[[0-9]{1,3}" +
                                    @"\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([a-zA-Z0-9\-]+\" +
                                    @".)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)$";
            var re = new Regex(strRegex);

            return re.IsMatch(email);

        }

        //Less restrictive. For examle admin@localhost will pass.
        public static bool IsEmail2(string email)
        {
            if (string.IsNullOrEmpty(email)) return false;

            const string strRegex = @"^([a-zA-Z0-9._-]+)@([a-zA-Z0-9.-]+)$";
            var re = new Regex(strRegex);

            return re.IsMatch(email);
        }

        public static bool IsMd5(string str)
        {
            if (string.IsNullOrEmpty(str)) return false;
            if (str.Length != 32) return false;
            if (!Regex.Match(str, "^[A-Fa-f0-9]{32}$").Success) return false;
            return true;
        }

        public static bool IsUrl(string url)
        {
            const string strRegex = "^(https?://)"
                                    + "?(([0-9a-z_!~*'().&=+$%-]+: )?[0-9a-z_!~*'().&=+$%-]+@)?" //user@ 
                                    + @"(([0-9]{1,3}\.){3}[0-9]{1,3}" // IP- 199.194.52.184 
                                    + "|" // allows either IP or domain 
                                    + @"([0-9a-z_!~*'()-]+\.)*" // tertiary domain(s)- www. 
                                    + @"([0-9a-z][0-9a-z-]{0,61})?[0-9a-z]\." // second level domain 
                                    + "[a-z]{2,6})" // first level domain- .com or .museum 
                                    + "(:[0-9]{1,4})?" // port number- :80 
                                    + "((/?)|" // a slash isn't required if there is no file name 
                                    + "(/[0-9a-z_!~*'().;?:@&=+$,%#-]+)+/?)$";
            var re = new Regex(strRegex);

            return re.IsMatch(url);
        }

        public static string ReplaceByRegExp(string input, string regularExpressionString, string replacement)
        {
            var re = new Regex(regularExpressionString);
            return re.Replace(input, replacement);
        }

        public static bool PatternMatch(string matchpattern, string subject)
        {
            var pat = matchpattern.Trim();
            if (string.IsNullOrEmpty(pat)) return false;

            if (pat.StartsWith("/") && pat.EndsWith("/"))
            {
                var pattern = pat.Substring(1, pat.Length - 2);
                try
                {
                    Regex rgx = new Regex(pattern, RegexOptions.IgnoreCase);
                    return (rgx.IsMatch(subject));
                }
                catch { }
            }

            return pat.ToLowerInvariant() == subject.ToLowerInvariant();
        }

        public static bool RegexMatch(string matchpattern, string subject)
        {
            var pat = matchpattern.Trim();
            if (string.IsNullOrEmpty(pat)) return false;

            string pattern = "";
            if (pat.StartsWith("/") && pat.EndsWith("/"))
            {
                pattern = pat.Substring(1, pat.Length - 2);
            }
            else
            {
                pattern = pat;
            }

            try
            {
                Regex rgx = new Regex(pattern, RegexOptions.IgnoreCase);
                return (rgx.IsMatch(subject));
            }
            catch
            {
            }

            return pat.ToLowerInvariant() == subject.ToLowerInvariant();
        }

    }
}
