﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {
        public static string Splithead(int sep, ref string target)
        {
            return Splithead(((char)sep).ToString(), ref target);
        }

        public static string Splithead(char sep, ref string target)
        {
            return Splithead(sep.ToString(), ref target);
        }

        public static string Splithead(string sep, ref string target)
        {
            var p1 = target.IndexOf(sep, StringComparison.Ordinal);
            if (p1 <= -1) return string.Empty;

            var ret = target.Substring(0, p1);
            target = target.Substring(p1 + sep.Length, target.Length - p1 - sep.Length);
            return ret;
        }

        public static char Splitchar(ref string target)
        {
            var ret = target[0];
            target = target.Substring(1, target.Length - 1);
            return (ret);
        }

        public static int SplitUint16(ref string target)
        {
            var byte1 = target[0];
            var byte2 = target[1];
            target = target.Substring(2, target.Length - 2);
            return (int)(byte1 << 8) + byte2;
        }

        public static string Splitnum(int num, ref string target)
        {
            if (num > target.Length)
            {
                num = target.Length;
            }
            var ret = target.Substring(0, num);
            var remlen = target.Length - num;
            target = remlen > 0 ? target.Substring(num, remlen) : string.Empty;
            return ret;
        }

        public static uint SplitUIntVal(ref string bytestring)
        {
            var onend = false;
            uint ret = 0;
            while ((!onend) && (bytestring.Length > 1))
            {
                int bt = Splitchar(ref bytestring);
                if ((bt & 128) == 0)
                {
                    onend = true;
                }
                int last7 = bt & 0x7F;
                ret = (uint)((((int)ret) << 7) | last7);
            }
            return ret;
        }

        public static uint SplitLittleEndianVal(ref string bytestring)
        {
            bool onend = false;
            uint ret = 0;
            while ((!onend) && (bytestring.Length > 0))
            {
                int bt = Splitchar(ref bytestring);
                if ((bt & 128) == 128)
                {
                    onend = true;
                }
                int last7 = bt & 0x7F;
                ret = (uint)((((int)ret) << 7) | last7);
            }
            return ret;
        }

        public static string ChopBytes(ref string pdu, int len)
        {
            if (pdu.Length < len)
            {
                len = pdu.Length;
            }
            string s = pdu.Substring(0, len);
            pdu = pdu.Substring(len, pdu.Length - len);
            return s;
        }

        public static char ChopChar(ref string pdu)
        {
            char s = pdu[0];
            pdu = pdu.Substring(1, pdu.Length - 1);
            return s;
        }

        public static string GetFrontWithDots(string source, int len)
        {
            if (string.IsNullOrEmpty(source)) return "";

            if (source.Length <= len) return source;
            else return GetFront(source, len, true) + "...";
        }

        public static string GetFront(string source, int len, bool returnIfShorter = false)
        {
            if (len > source.Length)
                return returnIfShorter ? source : String.Empty;

            return source.Substring(0,len);
        }

        public static string ChopFront(ref string source, int len)
        {
            if (len < 1) return source;
            if (len > source.Length)
                return "";

            var front = source.Substring(0, len);
            source = source.Substring(len, source.Length - len);
            return front;
        }

        public static string GetFront(string source, string sep)
        {
            var tochop = source;
            return ChopFront(ref tochop, sep);
        }

        public static string ChopFront(ref string source, string sep)
        {
            if (!source.Contains(sep))
            {
                var ret = source;
                source = "";
                return ret;
            }
            var p = source.IndexOf(sep);
            var head = source.Substring(0, p);
            var tail = source.Substring(p + sep.Length, source.Length - sep.Length-p);
            source = tail;
            return head;
        }


        public static string[] SeparateString(string input, char separationChar = ',')
        {
            var separatedStrings = new List<string>();
            var tmpStr = new StringBuilder();
            for (var i = 0; i < input.Length; ++i)
            {
                if (input[i] == separationChar)
                {
                    separatedStrings.Add(tmpStr.ToString());
                    tmpStr = tmpStr.Clear();
                }
                else
                    tmpStr.Append(input[i]);
            }
            if (tmpStr.ToString() != string.Empty)
                separatedStrings.Add(tmpStr.ToString());

            return separatedStrings.ToArray();
        }


        public static string Tail(string input, int length)
        {
            if (input == null) return String.Empty;

            return input.Length < length ? input : input.Substring(input.Length - length, length);
        }

        public static string ChopTrailingZero(string input)
        {
            if (string.IsNullOrEmpty(input)) { return string.Empty; }

            int x = input.Length;

            while ((--x > 0) && (input[x] == '\0' && input[x - 1] == '\0')) ;

            return input.Substring(0, x);
        }

        public static List<string> SplitToSize(string str, int chunkSize)
        {
            var list = Enumerable.Range(0, str.Length / chunkSize).Select(i => str.Substring(i * chunkSize, chunkSize)).ToList();
            var maradek = str.Substring(list.Count * chunkSize);
            list.Add(maradek);
            return list;
        }

        public static string ParseContentInPranthesis(string source, string findprefix, string tailchar = "\"")
        {
            var ind = source.IndexOf(findprefix, StringComparison.InvariantCultureIgnoreCase);
            if (ind > -1)
            {
                var tail = source.Substring(ind + findprefix.Length);
                var inde = tail.IndexOf(tailchar);
                if (inde > -1)
                {
                    var imgurl = tail.Substring(0, inde);
                    return imgurl;
                }
            }
            return null;
        }

        public static string ParseTagValue(string source, string tagname)
        {
            var p = source.IndexOf("<" + tagname + ">", StringComparison.InvariantCultureIgnoreCase);
            if (p == -1) return null;
            var tail = source.Substring(p + ("<" + tagname + ">").Length);
            var inde = tail.IndexOf("</" + tagname + ">", StringComparison.InvariantCultureIgnoreCase);
            if (inde == -1) return null;
            return tail.Substring(0, inde);
        }

        public static string[] ConvertToLines(string text)
        {
            var lines = text.Split(new string[] { "\r\n", "\r", "\n" }, StringSplitOptions.RemoveEmptyEntries);
            var ret = new List<string>(lines.Length);
            foreach (var l in lines)
            {
                var ln = Trim(l);
                if (ln == "") continue;
                ret.Add(ln);
            }
            return ret.ToArray();
        }
    }
}
