﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Globalization;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;

namespace Ozeki
{
    public static partial class OzCommonStrings
    {
        public static string HtmlDecode(string s)
        {
            return UrlDecode(s, Encoding.UTF8);
        }

        public static string UrlDecode(string s)
        {
            return UrlDecode(s, Encoding.UTF8);
        }

        public static string UrlDecode(string s, Encoding enc)
        {
            return OzHttpUtility.UrlDecode(s, enc);
        }

        public static string ReturnIt(string s)
        {
            return s;
        }

        public static string UrlEncode(string s)
        {
            return UrlEncode(s, Encoding.UTF8);
        }

        public static string UrlEncodeAsFileName(string s)
        {
            s = Regex.Replace(s.ToLower(CultureInfo.InvariantCulture), "\\s", "_");
            return UrlEncode(s, Encoding.UTF8);
        }

        public static string UrlEncode(string s, Encoding enc)
        {
            if (string.IsNullOrEmpty(s)) return "";
            return OzHttpUtility.UrlEncode(s, enc);
        }

        public static string UrlEncodeLight(string s)
        {
            return s.Replace("&", "%26").Replace("=", "%3d").Replace(" ", "+").Replace("<", "%3c");
        }

        public static string NameValueCollectionToQueryString(NameValueCollection nvc)
        {
            if (nvc == null) return "";
            bool first = true;
            var sb = new StringBuilder();
            foreach (string s in nvc)
                foreach (string v in nvc.GetValues(s))
                {
                    if (!first) sb.Append("&"); else first = false;
                    sb.Append(s);
                    sb.Append("=");
                    sb.Append(UrlEncode(v));
                }

            return sb.ToString();
        }

        public static string AppendTrailingSlash(string path)
        {
            if (string.IsNullOrEmpty(path)) return "/";
            if (!path.EndsWith("/")) return path + "/";
            return path;
        }


        public static string UrlAppendTrailingSlash(string url)
        {
            var urlWithoutPrefix = url.Replace("http://", "").Replace("https://", "");
            if (!urlWithoutPrefix.Contains("/")) url = url.Append("/");
            return url;
        }

        public static string AppendUrlParam(string url)
        {
            var urlWithoutPrefix = url.Replace("http://", "").Replace("https://", "");
            if (!urlWithoutPrefix.Contains("/")) url = url.Append("/");

            if (url.Contains("?"))
            {
                if (url.EndsWith("?")) return url;
                if (url.EndsWith("&")) return url;
                return url.Append("&");
            }
            if (url.EndsWith("&")) return url;
            return url.Append("?");
        }

        public static string ChangeHostnameInURL(string url, string newhostname)
        {
            var urlPrepared = Regex.Replace(url, "^(https?://)([^:/]*)([:/].*)$", m => string.Format("{0}{1}{2}", m.Groups[1].Value, newhostname, m.Groups[3].Value), RegexOptions.IgnoreCase);
            return urlPrepared;
        }

        public static string ChangeHostnameAndPortInURL(string url, string newhostnameandport)
        {
            var urlPrepared = Regex.Replace(url, "^(https?://)([^/]*)([/].*)$", m => string.Format("{0}{1}{2}", m.Groups[1].Value, newhostnameandport, m.Groups[3].Value), RegexOptions.IgnoreCase);
            return urlPrepared;
        }

        public static void DecodeQueryStringNameValuePair(string keypair, Dictionary<string, string> requestParamList, Encoding encoding)
        {
            if (string.IsNullOrEmpty(keypair)) return;

            var tempkeypair = keypair;
            if (tempkeypair == null || !tempkeypair.Contains("=")) return;

            var splitArray = tempkeypair.Split(_separator, 2);
            var key = splitArray[0];
            var value = splitArray[1];

            if ((!string.IsNullOrEmpty(key)) && (value != null))
            {
                key = OzHttpUtility.UrlDecode(key, encoding);
                value = OzHttpUtility.UrlDecode(value, encoding);

                if (requestParamList.ContainsKey(key))
                    requestParamList[key] = value;
                else
                    requestParamList.Add(key, value);
            }
        }

        public static void QueryStringToDictionary(string queryString, out Dictionary<string, string> result, Encoding encoding)
        {
            result = new Dictionary<string, string>();
            QueryStringToExistingDictionary(queryString, result, encoding);
        }

        public static void QueryStringToExistingDictionary(string queryString, Dictionary<string, string> result, Encoding encoding)
        {
            if (queryString.Contains("&"))
            {
                var split = queryString.Split('&');
                var length = split.Length;
                for (int i = 0; i < length; i++)
                    DecodeQueryStringNameValuePair(split[i], result, encoding);
            }
            else if (queryString.Contains("="))
            {
                DecodeQueryStringNameValuePair(queryString, result, encoding);
            }
        }

        public static string GetHost(string protocolHostPort)
        {
            GetHostPort(protocolHostPort, out string host, out int port);
            return host;
        }

        public static void GetHostPort(string hostandport, out string host, out int port)
        {
            var protocol = "";
            var ind = hostandport.IndexOf("://");
            if (ind>1)
            {
                protocol = hostandport.Substring(0, ind);
                hostandport = hostandport.Substring(ind+3);
                
            } 

            var portPos = hostandport.LastIndexOf(":");
            if (portPos == -1)
            {
                host = hostandport;
                port = 9522;
                return;
            }

            host = hostandport.Substring(0, portPos);
            var portstr = hostandport.Substring(portPos+1);

            try
            {
                port = int.Parse(portstr);
            }
            catch
            {
                port = 9522;
            }
        }


        public static void GetProtocolHostPort(string prothostandport, out string prot, out string host, out string port)
        {
            prot = "";
            host = "";
            port = "";
            var ind = prothostandport.IndexOf("://");
            if (ind > 1)
            {
                prot = prothostandport.Substring(0, ind);
                prothostandport = prothostandport.Substring(ind + 3);
            }

            var portPos = prothostandport.LastIndexOf(":");
            if (portPos == -1)
            {
                host = prothostandport;
                port = "";
                return;
            }

            host = prothostandport.Substring(0, portPos);
            port = prothostandport.Substring(portPos + 1);
        }

        public static void GetProtocolHostPath(string url, out string prot, out string host, out string path)
        {
            prot = "";
            host = "";
            path = "";

            var uri = new Uri(url);
            host = uri.Host;
            path = uri.PathAndQuery;

            if (string.IsNullOrEmpty(host))
            {
                prot = null;
                return;
            }
            var i = url.IndexOf(host);
            if (i==-1)
            {
                prot = null;
                return;
            }
            prot = url.Substring(0, i);
        }
    }
}