﻿using System;
using System.Globalization;
using System.Text.RegularExpressions;

namespace Ozeki
{
    public static partial class OzDateTime
    {
        static object syncObj = new object();
        static DateTime dateTime;
        static DateTime utcNow;
        static volatile string dateTimeString;

        //*******************************************************************
        // Construction
        //*******************************************************************
        static OzDateTime()
        {
            lastUpdate = Environment.TickCount - UPDATE_INTERVAL - 1;
            Update();
        }

        //*******************************************************************
        // Update
        //*******************************************************************
        const int UPDATE_INTERVAL = 800;
        static volatile int lastUpdate = Environment.TickCount;
        static volatile int totalSeconds;
        static volatile int totalSecondsUtc;
        public static int LastUpdate
        {
            get { return lastUpdate; }
        }

        static void Update()
        {
            int now = Environment.TickCount;
            if (!((now - lastUpdate > UPDATE_INTERVAL) || (now - lastUpdate < 0))) return;
            lock (syncObj)
            {
                if (!((now - lastUpdate > UPDATE_INTERVAL) || (now - lastUpdate < 0))) return;
                lastUpdate = now;
                dateTime = DateTime.Now;
                utcNow = DateTime.UtcNow;
                dateTimeString = dateTime.ToString(CultureInfo.InvariantCulture);
                dateTimeMySQLString = dateTime.ToString(MySQLFormat);
                totalSeconds = (int)DateTimeToUnix(DateTime.Now);
                totalSecondsUtc = (int)DateTimeToUnix(DateTime.UtcNow);
            }
        }

        //*******************************************************************
        // Now
        //*******************************************************************

        public static DateTime UtcNow
        {
            get
            {
                Update();
                return utcNow;
            }
        }

        public static DateTime Now
        {
            get
            {
                Update();
                return dateTime;
            }
        }

        public static string NowString
        {
            get
            {
                Update();
                return dateTimeString;
            }
        }

        public static TimeSpan UtcTimeZoneOffset
        {
            get { return TimeZone.CurrentTimeZone.GetUtcOffset(Now); }
        }

        public static long UnixNow
        {
            get
            {
                Update();
                return totalSeconds;
            }
        }

        public static long UnixNowUtc
        {
            get
            {
                Update();
                return totalSecondsUtc;
            }
        }

        public static string NowStringAsFileName
        {
            get
            {
                Update();
                return Regex.Replace(dateTimeMySQLString, @"[ .-/\\\\]", "_").Replace(":","-");
            }
        }

        public static DateTime RoundUp(DateTime dt, TimeSpan d)
        {
            return new DateTime((dt.Ticks + d.Ticks - 1) / d.Ticks * d.Ticks, dt.Kind);
        }

        //************************************************************
        // UTC
        //************************************************************
        public static DateTime FromUtcString(string str)
        {
            if (!DateTime.TryParse(str, out var dateTime)) return DateTime.Now;
            if (dateTime == DateTime.MinValue) return DateTime.MinValue;
            var dateTimeUtc = DateTime.SpecifyKind(dateTime, DateTimeKind.Utc);
            return dateTimeUtc.ToLocalTime();
        }

        public static string ToUtcString(DateTime dt)
        {
            if (dt == DateTime.MinValue) return dt.ToString(MySQLFormat);
            return dt.ToUniversalTime().ToString(MySQLFormat);
        }
    }
}
