﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net;
using System.Net.Mime;

namespace Ozeki
{
    public class OzHttpDownloadParams
    {

        //*************************************************
        // Request
        //*************************************************

        public string Url;
        public IOzProgressMonitor ProgressMonitor;
        public OzConf_HttpClient ProxyConfig;
        public string HttpBasicUsername;
        public string HttpBasicPassword;
        public string UserAgent;
        public object StateObject;
        public bool Async;
        public bool ResponseEncodingIsInHttpHeader;
        public int TimeoutInSec;
        public WebClientEx Client;
        public bool KeepAlive;
        public bool DisableCertificateValidation;
        public bool DisableExpect100Continue;
        public bool DisableAttachOzekiHeaders;
        public HTTPRequestMethod RequestMethod;
        public string PostData;
        public OzSslProtocolVersion SecurityProtocol;

        //*************************************************
        // Request headers
        //*************************************************

        Dictionary<string, string> _requestHeaders;
        public Dictionary<string, string> RequestHeaders
        {
            get { return _requestHeaders ?? (_requestHeaders = new Dictionary<string, string>()); }
            set { _requestHeaders = value; }
        }

        public const string RequestHeader_ContentType = "Content-Type";
        public const string RequestHeader_ContentType_Json = "application/json";
        public const string RequestHeader_ContentType_Form = "application/x-www-form-urlencoded";

        public void SetContentTypeToJSON()
        {
            if (RequestHeaders.ContainsKey(RequestHeader_ContentType))
                RequestHeaders.Remove(RequestHeader_ContentType);
            RequestHeaders.Add(RequestHeader_ContentType, RequestHeader_ContentType_Json);
        }

        public void SetContentTypeToForm()
        {
            if (RequestHeaders.ContainsKey(RequestHeader_ContentType))
                RequestHeaders.Remove(RequestHeader_ContentType);
            RequestHeaders.Add(RequestHeader_ContentType, RequestHeader_ContentType_Form);
        }

        public void SetContentType(string contentType)
        {
            if (RequestHeaders.ContainsKey(RequestHeader_ContentType))
                RequestHeaders.Remove(RequestHeader_ContentType);
            RequestHeaders.Add(RequestHeader_ContentType, contentType);
        }

        //*************************************************
        // Response
        //*************************************************

        public string ResponseUrl;
        public string ResponseString;
        public byte[] ResponseBytes;
        public string ResponseFilePath;

        Dictionary<string, string> _responseHeaders;
        public Dictionary<string, string> ResponseHeaders
        {
            get { return _responseHeaders ?? (_responseHeaders = new Dictionary<string, string>()); }
            set { _responseHeaders = value; }
        }

        public HttpStatusCode ResponseStatusCode;
        public bool IsSuccessResponseStatusCode
        {
            get { return ((int)ResponseStatusCode >= 200) && ((int)ResponseStatusCode <= 299); }
        }

        //*************************************************
        // Log
        //*************************************************

        public string Error;

        public string LogRequest;
        public string LogResponse;
        public string Log { get { return LogRequest + LogResponse; } }

        //*************************************************
        // Construction
        //*************************************************

        public OzHttpDownloadParams()
        {
            TimeoutInSec = 60 * 60;
            KeepAlive = true;
        }

        public override string ToString()
        {
            return Url ?? "";
        }
    }
}
