﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace Ozeki
{
    partial class OzHttpClient
    {
        //**************************************************************
        // Configure
        //**************************************************************

        static void configureWebClient(WebClientEx client, OzHttpDownloadParams downloadParams)
        {
            downloadParams.Client = client;

            setSecurityProtocol(downloadParams);
            setupHeaders(client, downloadParams);
            setupProxy(client, downloadParams);

            client.KeepAlive = downloadParams.KeepAlive;
            client.DisableExpect100Continue = downloadParams.DisableExpect100Continue;

            if (downloadParams.TimeoutInSec > 0)
                client.Timeout = downloadParams.TimeoutInSec * 1000;

            if (downloadParams.DisableCertificateValidation)
                disableCertificateValidation();
        }

        //**************************************************************
        // Headers
        //**************************************************************

        static void setupHeaders(WebClientEx client, OzHttpDownloadParams downloadParams)
        {
            if (string.IsNullOrEmpty(client.Headers["Content-Type"]))
                setupContentType(client, "application/octet-stream");
            
            setupOzekiHeaders(downloadParams);
            setupCustomHeaders(client.Headers, downloadParams);
            if (downloadParams.HttpBasicUsername == null || downloadParams.HttpBasicPassword == null) return;
            client.Headers[HttpRequestHeader.Authorization] = WriteBasicAuthentication(downloadParams.HttpBasicUsername, downloadParams.HttpBasicPassword);
        }

        public static string WriteBasicAuthentication(string username, string password)
        {
            string credentials = Convert.ToBase64String(Encoding.ASCII.GetBytes(username + ":" + password));
            return "Basic " + credentials;
        }

        static void setupContentType(WebClientEx client, string contentType)
        {
            client.Headers.Remove("Content-Type");
            client.Headers.Add("Content-Type", contentType);
        }

        //**************************************************************
        // Setup proxy
        //**************************************************************

        static void attachProxyConfig(HttpWebRequest request, OzConf_Proxy proxy)
        {
            if (proxy == null || !proxy.IsProxyEnabled) return;
            var credential = new NetworkCredential(proxy.ProxyUsername, proxy.ProxyPassword);
            var webProxy = new WebProxy(new Uri(proxy.ProxyHost + ":" + proxy.ProxyPort))
            {
                BypassProxyOnLocal = true,
                Credentials = credential
            };
            request.Proxy = webProxy;
        }

        static void setupProxy(WebClientEx client, OzHttpDownloadParams downloadParams)
        {
            if (downloadParams.ProxyConfig == null || !downloadParams.ProxyConfig.Proxy.IsProxyEnabled) return;
            client.Proxy = downloadParams.ProxyConfig.Proxy.ToWebProxy();
        }

        // 0 ref.
        static void setupProxy(WebClientEx client, OzHttpUploadParams uploadParams)
        {
            if (uploadParams.ProxyConfig == null || !uploadParams.ProxyConfig.Proxy.IsProxyEnabled) return;
            client.Proxy = uploadParams.ProxyConfig.Proxy.ToWebProxy();
        }

        //**************************************************************
        // Certificate validation
        //**************************************************************

        static bool ServicePointManagerValidationDisabled;

        static void disableCertificateValidation()
        {
            if (ServicePointManagerValidationDisabled) return;
            ServicePointManagerValidationDisabled = true;
            ServicePointManager.ServerCertificateValidationCallback += validateRemoteCertificate;
            //ServicePointManager.SecurityProtocol = SecurityProtocolType.Ssl3;
        }

        /// <summary>
        /// Certificate validation callback.
        /// </summary>
        static bool validateRemoteCertificate(object sender, X509Certificate cert, X509Chain chain, SslPolicyErrors error)
        {
            // If the certificate is a valid, signed certificate, return true.
            if (error == SslPolicyErrors.None) return true;

            OzLogger.Log(OzLogger.DefaultLogSource, LogLevel.Warning, "Invalid certificate received. " + cert.Subject + " " + error.ToString());
            return true;
        }
    }
}
