﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace Ozeki
{
    partial class OzHttpClient
    {
        //**************************************************************
        // Download
        //**************************************************************

        static void downloadProgressChanged(WebClientEx client, DownloadProgressChangedEventArgs e)
        {
            if (client.Progress == e.ProgressPercentage) return;
            client.Progress = e.ProgressPercentage;

            if (!(e.UserState is OzHttpDownloadParams downloadParams)) return;
            notifyMonitorAboutProgressChanged(downloadParams, client, e);
        }

        static void downloadCompleted(WebClientEx client, AsyncCompletedEventArgs e)
        {
            if (!(e.UserState is OzHttpDownloadParams downloadParams)) return;
            downloadParams.ResponseHeaders = extractResponseHeaders(client.ResponseHeaders);
            if (e.Error != null)
            {
                downloadParams.Error = e.Error.Message;
                if (e.Error.InnerException != null)
                    downloadParams.Error += " (" + e.Error.InnerException.Message + ")";
                notifyMonitorAboutDownloadCompleted(downloadParams);
                setSemaforAndDisposeClient(client);
                return;
            }
            if (e is DownloadStringCompletedEventArgs stringDownloadedEvent) // Download
                downloadParams.ResponseString = stringDownloadedEvent.Result;
            else if (e is DownloadDataCompletedEventArgs dataDownloadedEvent) // Download
                downloadParams.ResponseBytes = dataDownloadedEvent.Result;
            else if (e is UploadStringCompletedEventArgs stringUploadedEvent) // Upload
                downloadParams.ResponseString = stringUploadedEvent.Result;
            else if (e is UploadFileCompletedEventArgs dataUploadEvent) // Upload
                downloadParams.ResponseString = Encoding.UTF8.GetString(dataUploadEvent.Result);

            notifyMonitorAboutDownloadCompleted(downloadParams);
            setSemaforAndDisposeClient(client);
        }

        static void setSemaforAndDisposeClient(WebClientEx client)
        {
            client.Semafor.Set();
            client.Dispose();
        }

        //**************************************************************
        // Support functions
        //**************************************************************

        static void waitForCompletion(WebClientEx client, OzHttpDownloadParams downloadParams)
        {
            if (downloadParams.Async) return;

            if (downloadParams.TimeoutInSec > 0)
            {
                client.Semafor.WaitOne(downloadParams.TimeoutInSec * 1000 + 5000);
            }
            else
            {
                client.Semafor.WaitOne();
            }
        }
    }
}
