﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace Ozeki
{
    public static partial class OzHttpClient
    {
        //**************************************************************
        // Download data
        //**************************************************************
        public static bool HttpGetBytes(OzHttpDownloadParams downloadParams)
        {
            return HttpGetBytes(downloadParams, out var webClient);
        }

        public static bool HttpGetBytes(OzHttpDownloadParams downloadParams, out WebClientEx client)
        {
            downloadParams.Error = string.Empty;

            client = new WebClientEx();
            try
            {
                configureWebClient(client, downloadParams);
                wireUpEventsForDataDownload(client);

                notifyMonitorAboutDownloadStarted(downloadParams);
                client.DownloadDataAsync(new Uri(downloadParams.Url), downloadParams);
                
                waitForCompletion(client, downloadParams);

                if (!string.IsNullOrEmpty(downloadParams.Error)) return false;
                return true;
            }
            catch (Exception ex)
            {
                OzLogger.Log(OzLogSource.Default, LogLevel.Error, "Error occured while preparing for data download. Reason: " + ex.Message);
                client.Dispose();
                return false;
            }
        }

        //**************************************************************
        // Events / Callback
        //**************************************************************

        static void client_DownloadDataCompleted(object sender, DownloadDataCompletedEventArgs e)
        {
            if (!(sender is WebClientEx client)) return;
            wireDownDataDownloadEvents(client);
            downloadCompleted(client, e);
        }

        //**************************************************************
        // Events / Support
        //**************************************************************

        static void wireUpEventsForDataDownload(WebClientEx client)
        {
            client.DownloadDataCompleted += client_DownloadDataCompleted;
            client.DownloadProgressChanged += client_DownloadProgressChanged;
        }

        static void wireDownDataDownloadEvents(WebClientEx client)
        {
            client.DownloadDataCompleted -= client_DownloadDataCompleted;
            client.DownloadProgressChanged -= client_DownloadProgressChanged;
        }
    }
}
