﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace Ozeki
{
    public static partial class OzHttpClient
    {
        public static bool HttpDownloadFile(string URL, string saveToPath, out string error)
        {
            var downloadParams = new OzHttpDownloadParams();
            downloadParams.Url = URL;
            downloadParams.ResponseFilePath = saveToPath;
            HttpDownloadFile(downloadParams);
            error = downloadParams.Error;
            if (!string.IsNullOrEmpty(error)) return false;
            return true;
        }

        //**************************************************************
        // Download file
        //**************************************************************

        public static bool HttpDownloadFile(OzHttpDownloadParams downloadParams)
        {
            downloadParams.Error = string.Empty;

            var client = new WebClientEx();
            try
            {
                configureWebClient(client, downloadParams);
                wireUpEventsForFileDownload(client);

                notifyMonitorAboutDownloadStarted(downloadParams);
                client.DownloadFileAsync(new Uri(downloadParams.Url), downloadParams.ResponseFilePath, downloadParams);

                waitForCompletion(client, downloadParams);

                if (!string.IsNullOrEmpty(downloadParams.Error)) return false;
                return true;
            }
            catch (Exception ex)
            {
                OzLogger.Log(OzLogSource.Default, LogLevel.Error, "Error occured while preparing for file download. Reason: " + ex.Message);
                client.Dispose();
                return false;
            }
        }

        //**************************************************************
        // Events / Callback
        //**************************************************************

        static void client_DownloadProgressChanged(object sender, DownloadProgressChangedEventArgs e)
        {
            if (!(sender is WebClientEx client)) return;
            downloadProgressChanged(client, e);
        }

        static void client_DownloadFileCompleted(object sender, AsyncCompletedEventArgs e)
        {
            if (!(sender is WebClientEx client)) return;
            wireDownFileDownloadEvents(client);
            downloadCompleted(client, e);
        }

        //**************************************************************
        // Events / Support
        //**************************************************************

        static void wireUpEventsForFileDownload(WebClientEx client)
        {
            client.DownloadFileCompleted += client_DownloadFileCompleted;
            client.DownloadProgressChanged += client_DownloadProgressChanged;
        }

        static void wireDownFileDownloadEvents(WebClient client)
        {
            client.DownloadFileCompleted -= client_DownloadFileCompleted;
            client.DownloadProgressChanged -= client_DownloadProgressChanged;
        }
    }
}
