﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace Ozeki
{
    partial class OzHttpClient
    {
        //**************************************************************
        // Download string
        //**************************************************************

        static bool httpDownloadString(OzHttpDownloadParams downloadParams)
        {
            downloadParams.ResponseString = string.Empty;
            downloadParams.Error = string.Empty;

            var client = new WebClientEx();
            try
            {
                configureWebClient(client, downloadParams);
                wireUpEventsForDownloadString(client);

                notifyMonitorAboutDownloadStarted(downloadParams);
                client.DownloadStringAsync(new Uri(downloadParams.Url), downloadParams);

                waitForCompletion(client, downloadParams);

                if (!string.IsNullOrEmpty(downloadParams.Error)) return false;
                return true;
            }
            catch (Exception ex)
            {
                OzLogger.Log(OzLogSource.Default, LogLevel.Error, "Error occured while preparing for string download. Reason: " + ex.Message);
                client.Dispose();
                return false;
            }
        }

        //**************************************************************
        // Events / Callback
        //**************************************************************

        static void client_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (!(sender is WebClientEx client)) return;
            wireDownDownloadStringEvents(client);
            downloadCompleted(client, e);
        }

        //**************************************************************
        // Events / Support
        //**************************************************************

        static void wireUpEventsForDownloadString(WebClientEx client)
        {
            client.DownloadStringCompleted += client_DownloadStringCompleted;
            client.DownloadProgressChanged += client_DownloadProgressChanged;
        }

        static void wireDownDownloadStringEvents(WebClientEx client)
        {
            client.DownloadStringCompleted -= client_DownloadStringCompleted;
            client.DownloadProgressChanged -= client_DownloadProgressChanged;
        }
    }
}
