﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace Ozeki
{
    static partial class OzHttpClient
    {
        static void setServerCertificate(HttpWebRequest request, OzConf_HttpsClient config)
        {
            var acceptAllCert = false;
            if (config != null) acceptAllCert = config.SSLAcceptAllCertificates;
            //Másik szálon is befuthat egy másik request, config párt ezért nem tárolhatom el SSLAcceptAllCertificates változót 
            //statikusan OzHttpClient osztályon belül így külön osztályban tárolom az adott reqguest-hez tartozó configot.
            //Server Cert validálása nem azzonnal fut le.
            OzServerCertValidation.SetServerValidation(request, acceptAllCert);
        }

        static void setClientCertificate(HttpWebRequest request, OzConf_HttpsClient config)
        {
            if (config == null) return;
            if (string.IsNullOrEmpty(config.CertificatePath)) return;
            if (!File.Exists(config.CertificatePath)) return;
            var certificates = new X509Certificate2Collection();
            var flags = X509KeyStorageFlags.Exportable | X509KeyStorageFlags.MachineKeySet | X509KeyStorageFlags.PersistKeySet;
            certificates.Import(config.CertificatePath, config.CertificatePassword, flags);
            request.ClientCertificates = certificates;
        }

        static void setSecurityProtocol(OzHttpDownloadParams downParams)
        {
            if (!downParams.Url.ToLowerInvariant().StartsWith("https:")) return;

            var proxyConfig = downParams.ProxyConfig;
            if (proxyConfig != null && proxyConfig.HttpsClient != null)
                SetSecurityProtocol(downParams.ProxyConfig.HttpsClient.SSLVersion);
            else
                SetSecurityProtocol(downParams.SecurityProtocol);
        }

        public static void SetSecurityProtocol(OzSslProtocolVersion sslVersion)
        {
            switch (sslVersion)
            {
                case OzSslProtocolVersion.Default:
                    ServicePointManager.SecurityProtocol = SecurityProtocolType.Ssl3 | SecurityProtocolType.Tls | SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12;
                    break;
                case OzSslProtocolVersion.Ssl3:
                    ServicePointManager.SecurityProtocol = SecurityProtocolType.Ssl3;
                    break;
                case OzSslProtocolVersion.Tls10:
                    ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls;
                    break;
                case OzSslProtocolVersion.Tls11:
                    ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls11;
                    break;
                case OzSslProtocolVersion.Tls12:
                    ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12;
                    break;
            }
        }
    }
}
