﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace Ozeki
{
    partial class OzHttpClient
    {
        //**************************************************************
        // Support / Setup headers
        //**************************************************************

        public static void SetupHeaders(HttpWebRequest request, Dictionary<string, string> headers, OzLogSource logSource = null)
        {
            if (headers == null || headers.Count == 0) return;
            var headersCopy = new Dictionary<string, string>(headers, StringComparer.OrdinalIgnoreCase);
            setupPredefinedHeaders(request, headersCopy);
            setupCustomHeaders(request, headersCopy, logSource);
        }

        static void setupPredefinedHeaders(HttpWebRequest request, Dictionary<string, string> headers)
        {
            if (headers.ContainsKey("Host"))
            {
                request.Host = headers["Host"];
                headers.Remove("Host");
            }
            if (headers.ContainsKey("Content-Type"))
            {
                request.ContentType = headers["Content-Type"];
                headers.Remove("Content-Type");
            }
            if (headers.ContainsKey("Accept"))
            {
                request.Accept = headers["Accept"];
                headers.Remove("Accept");
            }
            if (headers.ContainsKey("Referer"))
            {
                request.Referer = headers["Referer"];
                headers.Remove("Referer");
            }
            if (headers.ContainsKey("User-Agent"))
            {
                request.UserAgent = headers["User-Agent"];
                headers.Remove("User-Agent");
            }
        }

        static void setupCustomHeaders(HttpWebRequest request, Dictionary<string, string> headers, OzLogSource logSource)
        {
            foreach (var header in headers)
            {
                try
                {
                    request.Headers.Remove(header.Key);
                    request.Headers.Add(header.Key, header.Value);
                }
                catch (Exception)
                {
                    var errorMessage = "HTTP request header set failed. " + header.Key + ": " + header.Value;
                    OzLogger.Log(logSource, LogLevel.Error, errorMessage);
                }
            }
        }

        //**************************************************************
        // Support / Authorization
        //**************************************************************

        static void setupHeaderAuthorization(HttpWebRequest request, OzHttpDownloadParams downloadParams)
        {
            if (string.IsNullOrEmpty(downloadParams.HttpBasicUsername)) return;
            if (string.IsNullOrEmpty(downloadParams.HttpBasicPassword)) return;
            var authorization = WriteBasicAuthentication(downloadParams.HttpBasicUsername, downloadParams.HttpBasicPassword);
            request.Headers.Add("Authorization", authorization);
        }

        
    }
}
