﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace Ozeki
{
    public static partial class OzHttpClient
    {
        //**************************************************************
        // Upload file
        //**************************************************************

        public static bool HttpUploadFile(OzHttpUploadParams uploadParams)
        {
            uploadParams.ResponseString = string.Empty;
            uploadParams.Error = string.Empty;

            var client = new WebClientEx();
            try
            {
                configureWebClient(client, uploadParams);
                client.StateObject = uploadParams.StateObject;
                wireUpEventsForFileUpload(client);

                notifyMonitorAboutDownloadStarted(uploadParams);
                var postUrl = setupPostUrl(uploadParams.Url, uploadParams.RequestParameters);
                client.UploadFileAsync(postUrl, "POST", uploadParams.FilePathToUpload, uploadParams);

                waitForCompletion(client, uploadParams);

                if (!string.IsNullOrEmpty(uploadParams.Error)) return false;
                return true;
            }
            catch (Exception ex)
            {
                OzLogger.Log(OzLogger.DefaultLogSource, LogLevel.Error, "Error occured while preparing for file upload. Reason: " + ex.Message);
                client.Dispose();
                return false;
            }
        }

        //**************************************************************
        // Events / Callback
        //**************************************************************

        private static void client_UploadFileCompleted(object sender, UploadFileCompletedEventArgs e)
        {
            if (!(sender is WebClientEx client)) return;
            wireDownFileUploadEvents(client);
            downloadCompleted(client, e);
        }

        //**************************************************************
        // Events / Support
        //**************************************************************

        static void wireUpEventsForFileUpload(WebClientEx client)
        {
            client.UploadFileCompleted += client_UploadFileCompleted;
            client.UploadProgressChanged += client_UploadProgressChanged;
        }

        static void wireDownFileUploadEvents(WebClientEx client)
        {
            client.UploadFileCompleted -= client_UploadFileCompleted;
            client.UploadProgressChanged -= client_UploadProgressChanged;
        }

        //**************************************************************
        // Support functions
        //**************************************************************

        static Uri setupPostUrl(string url, NameValueCollection requestParameters) 
        {
            var postUrl = OzCommonStrings.AppendUrlParam(url) + OzCommonStrings.NameValueCollectionToQueryString(requestParameters);
            return new Uri(postUrl);
        }
    }
}
