﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Globalization;
using System.IO;
using System.Runtime.InteropServices;

namespace Ozeki
{
    public static partial class OzHttpClient
    {
        //**************************************************************
        // Upload file
        //**************************************************************

        public static bool HttpUploadFiles(OzHttpUploadFilesParams uploadParams)
        {
            uploadParams.ResponseString = string.Empty;
            uploadParams.Error = string.Empty;

            var client = new WebClientEx();
            try
            {
                configureWebClient(client, uploadParams);
                client.StateObject = uploadParams.StateObject;
                wireUpEventsForFileUpload(client);

                notifyMonitorAboutDownloadStarted(uploadParams);
                var postUrl = setupPostUrl(uploadParams.Url, uploadParams.GetParameters);
                setBoundary(client, uploadParams);
                uploadFiles(client, postUrl, uploadParams);
                waitForCompletion(client, uploadParams);

                foreach (var responseHeaderKey in client.ResponseHeaders.AllKeys)
                {
                    if (uploadParams.ResponseHeaders.ContainsKey(responseHeaderKey)) continue;
                    uploadParams.ResponseHeaders.Add(responseHeaderKey, client.ResponseHeaders[responseHeaderKey]);
                }

                uploadParams.ResponseBytes = client.Response;
                uploadParams.ResponseString = client.Encoding.GetString(client.Response);

                if (!string.IsNullOrEmpty(uploadParams.Error)) return false;
                return true;
            }
            catch (Exception ex)
            {
                OzLogger.Log(OzLogSource.Default, LogLevel.Error, "Error occured while preparing for file upload. Reason: " + ex.Message);
                client.Dispose();
                return false;
            }
        }

        static void setBoundary(WebClientEx client, OzHttpUploadFilesParams uploadParams)
        {
            var boundary = "---------------------------" + DateTime.Now.Ticks.ToString("x", NumberFormatInfo.InvariantInfo);
            var contentType = "multipart/form-data; boundary=" + boundary;
            client.Headers.Remove("Content-Type");
            client.Headers.Add("Content-Type", contentType);
            boundary = "--" + boundary;
            uploadParams.Boundary = boundary;
        }

        static void uploadFiles(WebClientEx client, Uri postUrl, OzHttpUploadFilesParams uploadParams)
        {
            client.OpenWriteCompleted += Client_OpenWriteCompleted;
            client.DownloadDataCompleted += Client_DownloadDataCompleted;
            client.UploadProgressChanged += client_UploadProgressChanged;
            client.OpenWriteAsync(postUrl, "POST", uploadParams);
        }



        private static void Client_DownloadDataCompleted(object sender, DownloadDataCompletedEventArgs e)
        {

            var client = sender as WebClientEx;
            client.OpenWriteCompleted -= Client_OpenWriteCompleted;
            client.DownloadDataCompleted -= Client_DownloadDataCompleted;
            client.UploadProgressChanged -= client_UploadProgressChanged;
            downloadCompleted(client, e);
        }

        static void Client_OpenWriteCompleted(object sender, OpenWriteCompletedEventArgs e)
        {
            var uploadParams = e.UserState as OzHttpUploadFilesParams;
            var client = sender as WebClientEx;
            try
            {
                client.OpenWriteCompleted -= Client_OpenWriteCompleted;
                var requestStream = new MemoryStream();

                if (uploadParams.PostParameters != null)
                {
                    // Write the values
                    foreach (string name in uploadParams.PostParameters)
                    {
                        writePostData(name, requestStream, uploadParams);
                    }
                }

                // Write the files
                foreach (var file in uploadParams.FilesToUpload)
                {
                    writeFileData(file, requestStream, uploadParams);
                }

                var boundaryBuffer = Encoding.ASCII.GetBytes(uploadParams.Boundary + "--");
                requestStream.Write(boundaryBuffer, 0, boundaryBuffer.Length);

                var r = requestStream.ToArray();
                client.Headers.Add("Content-Length", r.Length.ToString());

                var serverStream = e.Result;
                serverStream.Write(r, 0, r.Length);
                serverStream.Close();
            }
            catch (Exception ex) 
            {
                uploadParams.Error = ex.Message;
                
            }
            client.Semafor.Set();
        }


        static void writePostData(string name, MemoryStream requestStream, OzHttpUploadFilesParams uploadParams)
        {
            var buffer = Encoding.ASCII.GetBytes(uploadParams.Boundary + Environment.NewLine);
            requestStream.Write(buffer, 0, buffer.Length);
            buffer = Encoding.ASCII.GetBytes(string.Format("Content-Disposition: form-data; name=\"{0}\"{1}{1}", name, Environment.NewLine));
            requestStream.Write(buffer, 0, buffer.Length);
            buffer = Encoding.UTF8.GetBytes(uploadParams.PostParameters[name] + Environment.NewLine);
            requestStream.Write(buffer, 0, buffer.Length);
        }

        static void writeFileData(OzHttpClientFileToUpload file, MemoryStream requestStream, OzHttpUploadFilesParams uploadParams)
        {
            var buffer = Encoding.ASCII.GetBytes(uploadParams.Boundary + Environment.NewLine);
            requestStream.Write(buffer, 0, buffer.Length);
            buffer = Encoding.UTF8.GetBytes(string.Format("Content-Disposition: form-data; name=\"{0}\"; filename=\"{1}\"{2}", file.Name, file.Filename, Environment.NewLine));
            requestStream.Write(buffer, 0, buffer.Length);
            buffer = Encoding.ASCII.GetBytes(string.Format("Content-Type: {0}{1}{1}", file.ContentType, Environment.NewLine));
            requestStream.Write(buffer, 0, buffer.Length);

            var bytes = File.ReadAllBytes(file.FullPath);
            requestStream.Write(bytes, 0, bytes.Length);

            buffer = Encoding.ASCII.GetBytes(Environment.NewLine);
            requestStream.Write(buffer, 0, buffer.Length);
        }
    }
}
