﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace Ozeki
{
    partial class OzHttpClient
    {
        //**************************************************************
        // Upload string
        //**************************************************************

        static bool httpUploadString(OzHttpDownloadParams downloadParams)
        {
            downloadParams.ResponseString = string.Empty;
            downloadParams.Error = string.Empty;

            var client = new WebClientEx();
            try
            {
                client.Headers[HttpRequestHeader.ContentType] = "application/x-www-form-urlencoded";
                configureWebClient(client, downloadParams);

                wireUpEventsForUploadString(client);
                client.UploadStringAsync(new Uri(downloadParams.Url), "POST", downloadParams.PostData, downloadParams);

                waitForCompletion(client, downloadParams);

                if (!string.IsNullOrEmpty(downloadParams.Error)) return false;
                return true;
            }
            catch (Exception ex)
            {
                OzLogger.Log(OzLogSource.Default, LogLevel.Error, "Error occured while preparing for string upload. Reason: " + ex.Message);
                downloadParams.Error = ex.Message;
                client.Dispose();
                return false;
            }
        }

        //**************************************************************
        // Events / Callback
        //**************************************************************

        static void client_UploadStringCompleted(object sender, UploadStringCompletedEventArgs e)
        {
            if (!(sender is WebClientEx client)) return;
            wireDownUploadStringEvents(client);
            downloadCompleted(client, e);
        }

        static void client_UploadProgressChanged(object sender, UploadProgressChangedEventArgs e)
        {
            if (!(sender is WebClientEx client)) return;
            uploadProgressChanged(client, e);
        }

        //**************************************************************
        // Events / Support
        //**************************************************************

        static void wireUpEventsForUploadString(WebClientEx client)
        {
            client.UploadStringCompleted += client_UploadStringCompleted;
            client.UploadProgressChanged += client_UploadProgressChanged;
        }

        static void wireDownUploadStringEvents(WebClientEx client)
        {
            client.UploadStringCompleted -= client_UploadStringCompleted;
            client.UploadProgressChanged -= client_UploadProgressChanged;
        }
    }
}
