﻿using System;
using System.Text;

namespace Ozeki
{
    public class OzLogMessage
    {
        public const string DateFormat = "yyyy-MM-dd HH:mm:ss.fff";

        //*************************************
        // Public members
        //*************************************

        public string TimeStamp;
        public LogLevel LogLevel;
        public string LogMessage;
        public string LogMessagePrefixed;
        public string LogPrefix;

        public OzLogSource LogSource;

        //*************************************
        // Construction
        //*************************************

        public OzLogMessage() { }

        public OzLogMessage(LogLevel level, string logMsg) : this(null, level, string.Empty, logMsg) { }

        public OzLogMessage(OzLogSource sender, LogLevel level, string prefix, string logMsg, bool includeEmptyLines = false)
        {
            TimeStamp = OzDateTime.Now.ToString(DateFormat);
            LogSource = sender;
            LogLevel = level;
            LogPrefix = prefix;
            LogMessage = logMsg.Replace('\0', ' ');  //A 0 byte, ami fájl vége karakter is egyeben sok problémát okozhat, fájlokban, adatbázis szerverekbe ezért kiszírjük

            if (string.IsNullOrEmpty(prefix)) return;

            var logEntry = new StringBuilder();
            string lineSepString = "\r";
            if (LogMessage.Contains("\r\n")) { lineSepString = "\r\n"; }
            string[] lines = LogMessage.Split(new[] { lineSepString }, StringSplitOptions.None);

            for (int x = 0; x < lines.Length; x++)
            {
                if ((x == lines.Length - 1) && !includeEmptyLines && (lines[x].Trim() == string.Empty)) continue;
                logLine(logEntry, lines[x]);
            }

            LogMessagePrefixed = logEntry.ToString().TrimEnd(OzCommonStrings.Cr, OzCommonStrings.Lf);
        }

        void logLine(StringBuilder logEntry, string ln)
        {
            var msg = new StringBuilder();
            msg.Append(LogPrefix);
            msg.Append(" ");
            msg.Append(ln);
            logEntry.AppendLine(ToFormatedLogLine(msg.ToString()));
        }

        string ToFormatedLogLine(string msg)
        {
            if (LogSource.Name == null) return null;
            var logLine = new StringBuilder();
            logLine.Append(TimeStamp);
            logLine.Append(" ");
            logLine.Append(LogLevelToString());
            logLine.Append(" ");
            logLine.Append(LogSource.Name.Replace(" ", "_"));
            logLine.Append(": ");
            logLine.Append(msg);
            return logLine.ToString();
        }

        public string LogLevelToString()
        {
            switch (LogLevel)
            {
                case LogLevel.Information: return "INFO";
                case LogLevel.Warning: return "WARN";
                case LogLevel.Error: return "ERRO";
                case LogLevel.Trace: return "TRAC";
                case LogLevel.Debug:
                default: return "DEBU";
            }
        }

        //*************************************
        // Return as string
        //*************************************

        public override string ToString()
        {
            if (!string.IsNullOrEmpty(LogPrefix))
                return LogMessagePrefixed;
            return ToFormatedLogLine(LogMessage);
        }

        //*************************************
        // Parse
        //*************************************

        public OzLogMessage Parse(string logLine)
        {
            try
            {
                if (logLine.Length < 1) return null;
                var index = logLine.IndexOf(' ', logLine.IndexOf(' ') + 1);

                TimeStamp = OzCommonStrings.ChopFront(ref logLine, index);  // 19
                OzCommonStrings.ChopFront(ref logLine, 1);
                var lev = OzCommonStrings.Splithead(' ', ref logLine);
                switch (lev)
                {
                    case "DEBU": LogLevel = LogLevel.Debug; break;
                    case "WARN": LogLevel = LogLevel.Warning; break;
                    case "ERRO": LogLevel = LogLevel.Error; break;
                    case "INFO": LogLevel = LogLevel.Information; break;
                    case "TRAC": LogLevel = LogLevel.Trace; break;
                    default: return null;
                }

                if (logLine.Contains(": "))
                {
                    var srcName = OzCommonStrings.Splithead(": ", ref logLine);
                    if (string.IsNullOrEmpty(srcName))
                        return null;

                    srcName = srcName.Replace("_", " ");

                    LogSource = OzLogSource.Create(srcName);
                    LogMessage = logLine;

                    return this;
                }

                return null;
            }
            catch { }
            return null;
        }

        //*************************************
        // Equals
        //*************************************

        public bool EqualsMessage(OzLogMessage obj)
        {
            return
                LogLevel == obj.LogLevel &&
                LogMessage == obj.LogMessage &&
                LogSource == obj.LogSource;
        }

        public override bool Equals(object obj)
        {
            if (!(obj is OzLogMessage message)) return false;
            return EqualsMessage(message) && TimeStamp == message.TimeStamp;
        }
    }
}