﻿using System;
using System.Runtime.Serialization;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace Ozeki
{
    [Serializable]
    public class OzLogSource
    {
        public static OzLogSource Default = OzLogSource.Create("Ozeki");

        //*************************************
        // Instance
        //*************************************

        public string Name;

        public OzLogMessage previousMsg;
        public int _count = 0;

        OzLogSource(string logSourceName)
        {
            Name = logSourceName;
        }

        public override string ToString()
        {
            return "Logsource: " + Name;
        }

        //*************************************
        // Create
        //*************************************

        static readonly object syncLogSourceCreate = new object();
        static Dictionary<string, OzLogSource> _registeredSources = new Dictionary<string, OzLogSource>();

        public static OzLogSource Create(string name)
        {
            lock (syncLogSourceCreate)
            {
                if (string.IsNullOrEmpty(name)) name = "DefaultLog";
                name = name.Replace('\\', '/');
                if (_registeredSources.ContainsKey(name))
                    return _registeredSources[name];

                var newLogSource = new OzLogSource(name);
                _registeredSources.Add(name, newLogSource);
                return newLogSource;
            }
        }

        public static bool Exists(string name, out OzLogSource logSource)
        {
            lock (syncLogSourceCreate)
            {
                logSource = null;
                if (string.IsNullOrEmpty(name)) return false;
                if (!_registeredSources.ContainsKey(name)) return false;
                logSource = _registeredSources[name];
                return true;
            }
        }

        public static string[] List()
        {
            lock (syncLogSourceCreate)
            {
                return _registeredSources.Keys.ToArray();
            }
        }

        public static void Unregister(OzLogSource logSource)
        {
            lock (syncLogSourceCreate)
            {
                _registeredSources.Remove(logSource.Name);
            }
        }
    }
}